# test_WhiteWolfParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Test the white wolf card reader"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.core.SutekhObjects import AbstractCard, IAbstractCard, \
        IPhysicalCard, IClan, IDisciplinePair, ICardType, ISect, ITitle, \
        ICreed, IVirtue, IExpansion, IRarity, IRarityPair, IArtist, IKeyword
from sqlobject import SQLObjectNotFound
import unittest


class WhiteWolfParserTests(SutekhTest):
    """Test class for the white wolf card reader.

       Check the parsing done in SutekhTest setup and verify the results
       are correct.
       """
    aExpectedCards = [
        u".44 Magnum",
        u"AK-47",
        u"Aabbt Kindred",
        u"Aaron Bathurst",
        u"Aaron Duggan, Cameron's Toady",
        u"Aaron's Feeding Razor",
        u"Abandoning the Flesh",
        u"Abbot",
        u"Abd al-Rashid",
        u"Abdelsobek",
        u"Abebe",
        u"Abjure",
        u"Ablative Skin",
        u"Abombwe",
        u"Aeron",
        u"Agent of Power",
        u"Aire of Elation",
        u"Akram",
        u"Alan Sovereign",
        u"Alan Sovereign (Advanced)",
        u"Alexandra",
        u"Alfred Benezri",
        u"Ambrogino Giovanni",
        u'Amisa',
        u'Anarch Railroad',
        u'Anarch Revolt',
        u"Anastasz di Zagreb",
        u"Angelica, The Canonicus",
        u'Anna "Dictatrix11" Suljic',
        u"Anson",
        u"Bravo",
        u"Bronwen",
        u"Cedric",
        u"Cesewayo",
        u'Dramatic Upheaval',
        u'Earl "Shaka74" Deams',
        u"Enkidu, The Noah",
        u"Fidus, The Shrunken Beast",
        u"Ghoul Retainer",
        u"Gracis Nostinus",
        u"Gypsies",
        u'Hide the Heart',
        u"High Top",
        u'Inez "Nurse216" Villagrande',
        u"Kabede Maru",
        u"Kemintiri (Advanced)",
        u"Living Manse",
        u"L\xe1z\xe1r Dobrescu",
        u'Motivated by Gehenna',
        u"Necromancy",
        u"Ossian",
        u"Pariah",
        u"Paris Opera House",
        u"Park Hunting Ground",
        u"Pier 13, Port of Baltimore",
        u"Political Hunting Ground",
        u"Predator's Communion",
        u"Protracted Investment",
        u"Raven Spy",
        u"Rebekka, Chantry Elder of Munich",
        u"Rock Cat",
        u'Scapelli, The Family "Mechanic"',
        u"Sha-Ennu",
        u"Shade",
        u"Smite",
        u"The Ankara Citadel, Turkey",
        u"The Path of Blood",
        u"The Siamese",
        u"The Slaughterhouse",
        u"Two Wrongs",
        u"Vox Domini",
        u"Yvette, The Hopeless",
        ]

    def test_basic(self):
        """Basic WW list parser tests"""
        # pylint: disable-msg=E1101, R0915, R0914
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # R0915, R0914: Want a long, sequential test case to minimise
        aCards = sorted(list(AbstractCard.select()), cmp=lambda oC1, oC2:
                cmp(oC1.name, oC2.name))

        # Check card names
        self.assertEqual([oC.name for oC in aCards], self.aExpectedCards)

        # Check Magnum
        # pylint: disable-msg=C0103
        # o44 is OK here
        o44 = IAbstractCard(".44 Magnum")
        self.assertEqual(o44.canonicalName, u".44 magnum")
        self.assertEqual(o44.name, u".44 Magnum")
        self.failUnless(o44.text.startswith(u"Weapon, gun.\n2R"))
        self.failUnless(o44.text.endswith(u"each combat."))
        self.assertEqual(o44.group, None)
        self.assertEqual(o44.capacity, None)
        self.assertEqual(o44.cost, 2)
        self.assertEqual(o44.life, None)
        self.assertEqual(o44.costtype, 'pool')
        self.assertEqual(o44.level, None)

        # pylint: enable-msg=C0103
        oCommon = IRarity('Common')
        oJyhad = IExpansion('Jyhad')
        oVTES = IExpansion('VTES')

        self.assertTrue(oCommon in [oP.rarity for oP in o44.rarity])
        self.assertTrue(oJyhad in [oP.expansion for oP in o44.rarity])
        self.assertTrue(oVTES in [oP.expansion for oP in o44.rarity])

        self.assertTrue(IRarityPair(('VTES', 'Common')) in o44.rarity)

        self.failUnless(IKeyword('gun') in o44.keywords)
        self.failUnless(IKeyword('weapon') in o44.keywords)

        # Find some discipline pairs
        oFortInf = IDisciplinePair((u"Fortitude", u"inferior"))
        oFortSup = IDisciplinePair((u"Fortitude", u"superior"))
        oQuiSup = IDisciplinePair((u"Quietus", u"superior"))
        oCelInf = IDisciplinePair((u"Celerity", u"inferior"))
        oAusInf = IDisciplinePair((u"Auspex", u"inferior"))
        oAusSup = IDisciplinePair((u"Auspex", u"superior"))
        oPreSup = IDisciplinePair((u"Presence", u"superior"))
        oObfSup = IDisciplinePair((u"Obfuscate", u"superior"))
        oAboInf = IDisciplinePair((u"Abombwe", u"inferior"))
        oAboSup = IDisciplinePair((u"Abombwe", u"superior"))
        oValSup = IDisciplinePair((u'Valeren', u"superior"))

        # Check Dobrescu
        oDob = IAbstractCard(u"L\xe1z\xe1r Dobrescu")
        self.assertEqual(oDob.canonicalName, u"l\xe1z\xe1r dobrescu")
        self.assertEqual(oDob.name, u"L\xe1z\xe1r Dobrescu")
        self.failUnless(oDob.text.startswith(
            u"Independent: L\xe1z\xe1r may move"))
        self.failUnless(oDob.text.endswith(u"as a (D) action."))
        self.assertEqual(oDob.group, 2)
        self.assertEqual(oDob.capacity, 3)
        self.assertEqual(oDob.cost, None)
        self.assertEqual(oDob.life, None)
        self.assertEqual(oDob.costtype, None)
        self.assertEqual(oDob.level, None)

        self.failUnless(IClan('Ravnos') in oDob.clan)
        self.assertEqual(len(oDob.discipline), 1)
        self.failUnless(oFortInf in oDob.discipline)
        self.assertEqual(len(oDob.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oDob.cardtype)
        self.failUnless(ISect('Independent') in oDob.sect)
        self.assertEqual(len(oDob.artists), 1)
        self.failUnless(IArtist('Rebecca Guay') in oDob.artists)

        self.assertEqual(len(oDob.rulings), 1)
        oRuling = oDob.rulings[0]
        self.assertTrue(oRuling.text.startswith("Cannot use his special"))
        self.assertTrue(oRuling.text.endswith("uncontrolled region."))
        self.assertEqual(oRuling.code, "[LSJ 19990215]")

        # Check Abstract and Physical expansions match
        for oAbs in AbstractCard.select():
            aExps = [oPair.expansion for oPair in oAbs.rarity]
            for oExp in aExps:
                try:
                    oPair = IPhysicalCard((oAbs, oExp))
                except SQLObjectNotFound:
                    self.fail("Missing physical card %s from expansion %s"
                        % (oAbs.name, oExp.name))

        # Check Yvette
        oYvette = IAbstractCard(u"Yvette, The Hopeless")
        self.assertEqual(oYvette.canonicalName, u"yvette, the hopeless")
        self.assertEqual(oYvette.name, u"Yvette, The Hopeless")
        self.failUnless(oYvette.text.startswith("Camarilla."))
        self.assertEqual(oYvette.group, 3)
        self.assertEqual(oYvette.capacity, 3)
        self.assertEqual(oYvette.cost, None)
        self.assertEqual(oYvette.life, None)
        self.assertEqual(oYvette.costtype, None)
        self.assertEqual(oYvette.level, None)

        self.failUnless(IClan('Toreador') in oYvette.clan)
        self.assertEqual(len(oYvette.clan), 1)
        self.assertEqual(len(oYvette.discipline), 2)
        self.failUnless(oFortInf not in oYvette.discipline)
        self.failUnless(oCelInf in oYvette.discipline)
        self.failUnless(oAusInf in oYvette.discipline)
        self.assertEqual(len(oYvette.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oYvette.cardtype)
        self.failUnless(ISect('Camarilla') in oYvette.sect)
        self.assertEqual(len(oYvette.artists), 1)
        self.failUnless(IArtist('Leif Jones') in oYvette.artists)

        # Check Sha-Ennu

        oShaEnnu = IAbstractCard(u"Sha-Ennu")
        self.assertEqual(oShaEnnu.canonicalName, u"sha-ennu")
        self.assertEqual(oShaEnnu.name, u"Sha-Ennu")
        self.failUnless(oShaEnnu.text.startswith("Sabbat regent:"))
        self.failUnless(oShaEnnu.text.endswith("+2 bleed."))
        self.assertEqual(oShaEnnu.group, 4)
        self.assertEqual(oShaEnnu.capacity, 11)
        self.assertEqual(oShaEnnu.cost, None)
        self.assertEqual(oShaEnnu.life, None)
        self.assertEqual(oShaEnnu.costtype, None)
        self.assertEqual(oShaEnnu.level, None)

        self.failUnless(IClan('Tzimisce') in oShaEnnu.clan)
        self.assertEqual(len(oShaEnnu.clan), 1)
        self.assertEqual(len(oShaEnnu.discipline), 6)
        self.failUnless(oAusSup in oShaEnnu.discipline)
        self.failUnless(oAusInf not in oShaEnnu.discipline)
        self.assertEqual(len(oShaEnnu.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oShaEnnu.cardtype)
        self.failUnless(ISect('Sabbat') in oShaEnnu.sect)
        self.failUnless(ITitle('Regent') in oShaEnnu.title)
        self.assertTrue(IRarityPair(('Third', 'Vampire')) in oShaEnnu.rarity)
        self.assertFalse(IRarityPair(('VTES', 'Common')) in oShaEnnu.rarity)
        self.assertFalse(IRarityPair(('VTES', 'Vampire')) in oShaEnnu.rarity)
        self.assertEqual(len(oShaEnnu.artists), 1)
        self.failUnless(IArtist('Richard Thomas') in oShaEnnu.artists)

        self.failUnless(IKeyword('0 stealth') in oShaEnnu.keywords)
        self.failUnless(IKeyword('0 intercept') in oShaEnnu.keywords)
        self.failUnless(IKeyword('1 strength') in oShaEnnu.keywords)
        self.failUnless(IKeyword('3 bleed') in oShaEnnu.keywords)

        # Check Kabede

        oKabede = IAbstractCard(u"Kabede Maru")
        self.assertEqual(oKabede.canonicalName, u"kabede maru")
        self.assertEqual(oKabede.name, u"Kabede Maru")
        self.failUnless(oKabede.text.startswith("Laibon magaji:"))
        self.failUnless(oKabede.text.endswith("affect Kabede.)"))
        self.assertEqual(oKabede.group, 5)
        self.assertEqual(oKabede.capacity, 9)
        self.assertEqual(oKabede.cost, None)
        self.assertEqual(oKabede.life, None)
        self.assertEqual(oKabede.costtype, None)
        self.assertEqual(oKabede.level, None)

        self.failUnless(IClan('Assamite') in oKabede.clan)
        self.assertEqual(len(oKabede.clan), 1)
        self.assertEqual(len(oKabede.discipline), 6)
        self.failUnless(oAusSup in oKabede.discipline)
        self.failUnless(oQuiSup in oKabede.discipline)
        self.failUnless(oAboInf in oKabede.discipline)
        self.failUnless(oAboSup not in oKabede.discipline)
        self.assertEqual(len(oKabede.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oKabede.cardtype)
        self.failUnless(ISect('Laibon') in oKabede.sect)
        self.failUnless(ITitle('Magaji') in oKabede.title)
        self.assertTrue(IRarityPair(('LotN', 'Uncommon')) in oKabede.rarity)
        self.assertEqual(len(oKabede.artists), 1)
        self.failUnless(IArtist('Ken Meyer, Jr.') in oKabede.artists)

        self.failUnless(IKeyword('0 stealth') in oKabede.keywords)
        self.failUnless(IKeyword('0 intercept') in oKabede.keywords)
        self.failUnless(IKeyword('1 strength') in oKabede.keywords)
        self.failUnless(IKeyword('1 bleed') in oKabede.keywords)

        # Check Predator's Communion
        oPredComm = IAbstractCard(u"Predator's Communion")
        self.assertEqual(oPredComm.canonicalName, u"predator's communion")
        self.assertEqual(oPredComm.name, u"Predator's Communion")
        self.failUnless(oPredComm.text.startswith("[abo] [REFLEX]"))
        self.failUnless(oPredComm.text.endswith("untaps."))
        self.assertEqual(oPredComm.group, None)
        self.assertEqual(oPredComm.capacity, None)
        self.assertEqual(oPredComm.cost, None)
        self.assertEqual(oPredComm.life, None)
        self.assertEqual(oPredComm.costtype, None)
        self.assertEqual(oPredComm.level, None)

        self.assertEqual(len(oPredComm.discipline), 1)
        self.failUnless(oAboSup in oPredComm.discipline)
        self.assertEqual(len(oPredComm.cardtype), 2)
        self.failUnless(ICardType('Reaction') in oPredComm.cardtype)
        self.failUnless(ICardType('Reflex') in oPredComm.cardtype)
        self.assertTrue(IRarityPair(('LoB', 'Common')) in oPredComm.rarity)
        self.assertEqual(len(oPredComm.artists), 1)
        self.failUnless(IArtist('David Day') in oPredComm.artists)

        # Check Earl
        oEarl = IAbstractCard(u'Earl "Shaka74" Deams')
        self.assertEqual(oEarl.canonicalName, u'earl "shaka74" deams')
        self.assertEqual(oEarl.name, u'Earl "Shaka74" Deams')
        self.failUnless(oEarl.text.startswith("Earl gets +1 stealth"))
        self.failUnless(oEarl.text.endswith("[1 CONVICTION]."))
        self.assertEqual(oEarl.group, 4)
        self.assertEqual(oEarl.capacity, None)
        self.assertEqual(oEarl.cost, None)
        self.assertEqual(oEarl.life, 6)
        self.assertEqual(oEarl.costtype, None)
        self.assertEqual(oEarl.level, None)

        self.failUnless(ICreed('Visionary') in oEarl.creed)
        self.assertEqual(len(oEarl.creed), 1)
        self.assertEqual(len(oEarl.virtue), 3)
        self.failUnless(IVirtue('Martyrdom') in oEarl.virtue)
        self.failUnless(IVirtue('Judgment') in oEarl.virtue)
        self.failUnless(IVirtue('Vision') in oEarl.virtue)
        self.assertEqual(len(oEarl.cardtype), 1)
        self.failUnless(ICardType('Imbued') in oEarl.cardtype)
        self.assertTrue(IRarityPair(('NoR', 'Uncommon')) in oEarl.rarity)
        self.assertEqual(len(oEarl.artists), 1)
        self.failUnless(IArtist('David Day') in oEarl.artists)

        # Check Aire
        oAire = IAbstractCard("Aire of Elation")
        self.assertEqual(oAire.canonicalName, u"aire of elation")
        self.assertEqual(oAire.name, u"Aire of Elation")
        self.failUnless(oAire.text.startswith("You cannot play another"))
        self.failUnless(oAire.text.endswith("is Toreador."))
        self.assertEqual(oAire.group, None)
        self.assertEqual(oAire.capacity, None)
        self.assertEqual(oAire.cost, 1)
        self.assertEqual(oAire.life, None)
        self.assertEqual(oAire.costtype, 'blood')
        self.assertEqual(oAire.level, None)

        self.assertEqual(len(oAire.discipline), 1)
        self.failUnless(oPreSup in oAire.discipline)
        self.assertEqual(len(oAire.cardtype), 1)
        self.failUnless(ICardType("Action Modifier") in oAire.cardtype)

        self.assertTrue(IRarityPair(('DS', 'Common')) in oAire.rarity)
        self.assertTrue(IRarityPair(('FN', 'Precon')) in oAire.rarity)
        self.assertTrue(IRarityPair(('CE', 'Common')) in oAire.rarity)
        self.assertTrue(IRarityPair(('CE', 'Precon')) in oAire.rarity)
        self.assertTrue(IRarityPair(('Anarchs', 'Precon')) in oAire.rarity)
        self.assertTrue(IRarityPair(('KMW', 'Precon')) in oAire.rarity)

        self.assertEqual(len(oAire.artists), 1)
        self.failUnless(IArtist('Greg Simanson') in oAire.artists)

        # Abjure
        oAbjure = IAbstractCard("Abjure")
        self.assertEqual(oAbjure.canonicalName, u"abjure")
        self.assertEqual(oAbjure.name, u"Abjure")
        self.failUnless(oAbjure.text.startswith("[COMBAT] Tap this"))
        self.failUnless(oAbjure.text.endswith("or ash heap."))
        self.assertEqual(oAbjure.group, None)
        self.assertEqual(oAbjure.capacity, None)
        self.assertEqual(oAbjure.cost, None)
        self.assertEqual(oAbjure.life, None)
        self.assertEqual(oAbjure.costtype, None)
        self.assertEqual(oAbjure.level, None)

        self.assertEqual(len(oAbjure.virtue), 1)
        self.failUnless(IVirtue('Redemption') in oAbjure.virtue)
        self.assertEqual(len(oAbjure.cardtype), 1)
        self.failUnless(ICardType('Power') in oAbjure.cardtype)
        self.assertEqual(len(oAbjure.artists), 1)
        self.failUnless(IArtist('Brian LeBlanc') in oAbjure.artists)

        # Abombwe Master Card
        oAbo = IAbstractCard('Abombwe')

        self.assertEqual(oAbo.canonicalName, 'abombwe')
        self.assertEqual(oAbo.name, 'Abombwe')
        self.failUnless(oAbo.text.startswith("Master: Discipline. Trifle"))
        self.failUnless(oAbo.text.endswith("superior Abombwe."))
        self.assertEqual(oAbo.group, None)
        self.assertEqual(oAbo.capacity, 1)
        self.assertEqual(oAbo.cost, None)
        self.assertEqual(oAbo.life, None)
        self.assertEqual(oAbo.costtype, None)
        self.assertEqual(oAbo.level, None)

        self.assertEqual(len(oAbo.discipline), 0)
        self.assertEqual(len(oAbo.cardtype), 1)
        self.failUnless(ICardType('Master') in oAbo.cardtype)
        self.assertEqual(len(oAbo.rulings), 0)
        self.assertEqual(len(oAbo.artists), 1)
        self.failUnless(IArtist('Ken Meyer, Jr.') in oAbo.artists)

        self.failUnless(IKeyword('trifle') in oAbo.keywords)
        self.failUnless(IKeyword('discipline') in oAbo.keywords)

        # Ablative Skin Card
        oAblat = IAbstractCard('Ablative Skin')

        self.assertEqual(oAblat.canonicalName, 'ablative skin')
        self.assertEqual(oAblat.name, 'Ablative Skin')
        self.failUnless(oAblat.text.startswith("+1 stealth action"))
        self.failUnless(oAblat.text.endswith("damage in combat in this way."))
        self.assertEqual(oAblat.group, None)
        self.assertEqual(oAblat.cost, None)
        self.assertEqual(oAblat.life, None)
        self.assertEqual(oAblat.costtype, None)
        self.assertEqual(oAblat.level, None)

        self.assertEqual(len(oAblat.discipline), 1)
        self.failUnless(oFortSup in oAblat.discipline)
        self.assertEqual(len(oAblat.cardtype), 1)
        self.failUnless(ICardType('Action') in oAblat.cardtype)
        self.assertEqual(len(oAblat.rulings), 1)
        oRuling = oAblat.rulings[0]
        self.assertTrue(oRuling.text.startswith("Cannot be used to prevent"))
        self.assertTrue(oRuling.text.endswith("Blood Fury)."))
        self.assertEqual(oRuling.code, "[LSJ 19990216]")
        self.assertEqual(len(oAblat.artists), 1)
        self.failUnless(IArtist('Richard Thomas') in oAblat.artists)

        # Check Independent titles
        oAmbrig = IAbstractCard(u"Ambrogino Giovanni")
        self.failUnless(ITitle('Independent with 1 vote') in oAmbrig.title)
        oAmisa = IAbstractCard(u"Amisa")
        self.failUnless(ITitle('Independent with 2 votes') in oAmisa.title)

        # Check Kemintiri
        oKemintiri = IAbstractCard(u"Kemintiri (Advanced)")
        self.assertEqual(oKemintiri.canonicalName, u"kemintiri (advanced)")
        self.assertEqual(oKemintiri.name, u"Kemintiri (Advanced)")
        self.failUnless(oKemintiri.text.startswith("Advanced, Independent."))
        self.assertEqual(oKemintiri.group, 2)
        self.assertEqual(oKemintiri.capacity, 10)
        self.assertEqual(oKemintiri.cost, None)
        self.assertEqual(oKemintiri.life, None)
        self.assertEqual(oKemintiri.costtype, None)
        self.assertEqual(oKemintiri.level, 'advanced')
        self.failUnless(IKeyword('advanced') in oKemintiri.keywords)

        self.failUnless(IClan('Follower of Set') in oKemintiri.clan)
        self.assertEqual(len(oKemintiri.clan), 1)
        self.assertEqual(len(oKemintiri.discipline), 6)
        self.failUnless(oAusInf in oKemintiri.discipline)
        self.failUnless(oObfSup in oKemintiri.discipline)
        self.failUnless(oPreSup in oKemintiri.discipline)
        self.assertEqual(len(oKemintiri.cardtype), 1)
        self.failUnless(ICardType('Vampire') in oKemintiri.cardtype)
        self.failUnless(ISect('Independent') in oKemintiri.sect)
        self.assertEqual(len(oKemintiri.artists), 1)
        self.failUnless(IArtist('Lawrence Snelly') in oKemintiri.artists)

        # Make sure we're not picking up the Merged title
        self.assertEqual(len(oKemintiri.title), 0)
        self.assertTrue(IRarityPair(('KMW', 'Uncommon')) in oKemintiri.rarity)

        # Check The Path
        oPath1 = IAbstractCard('The Path of Blood')

        self.failUnless(IClan('Assamite') in oPath1.clan)
        self.assertEqual(oPath1.cost, 1)
        self.assertEqual(oPath1.costtype, 'pool')
        self.failUnless(ICardType('Master') in oPath1.cardtype)

        self.failUnless(IKeyword('unique') in oPath1.keywords)

        # Check alternative lookup
        oPath2 = IAbstractCard('Path of Blood, The')
        self.assertEqual(oPath1.id, oPath2.id)

        # Check The Slaughterhouse
        oSlaughter = IAbstractCard('The Slaughterhouse')
        self.assertEqual(oSlaughter.canonicalName, u"the slaughterhouse")
        self.assertEqual(len(oSlaughter.keywords), 2)
        self.failUnless(IKeyword('burn option') in oSlaughter.keywords)
        self.failUnless(IKeyword('location') in oSlaughter.keywords)

        # Check keywords for The Ankara Citadel, Turkey
        oCitadel = IAbstractCard('The Ankara Citadel, Turkey')
        self.assertEqual(oCitadel.cost, 2)
        self.failUnless(IClan('Tremere') in oCitadel.clan)
        self.assertEqual(oCitadel.costtype, 'blood')
        self.failUnless(IKeyword('location') in oCitadel.keywords)
        self.failUnless(IKeyword('unique') in oCitadel.keywords)

        # Check life & keywords for Ossian
        oOssian = IAbstractCard('Ossian')
        self.assertEqual(oOssian.life, 4)
        self.assertEqual(oOssian.cost, 3)
        self.assertEqual(oOssian.costtype, 'pool')
        self.failUnless(IKeyword('werewolf') in oOssian.keywords)
        self.failUnless(IKeyword('red list') in oOssian.keywords)
        self.failUnless(IKeyword('unique') in oOssian.keywords)
        self.failUnless(IKeyword('0 bleed') in oOssian.keywords)
        self.failUnless(IKeyword('2 strength') in oOssian.keywords)

        # Check life & keywords for Raven Spy
        oRaven = IAbstractCard('Raven Spy')
        self.failUnless(IKeyword('animal') in oRaven.keywords)
        self.failUnless(IKeyword('1 strength') not in oRaven.keywords)
        self.assertEqual(oRaven.life, 1)
        self.assertEqual(oRaven.cost, 1)
        self.assertEqual(oRaven.costtype, 'blood')

        # Check special cases
        oRetainer = IAbstractCard('Ghoul Retainer')
        self.failUnless(IKeyword('1 strength') in oRetainer.keywords)
        oHighTop = IAbstractCard('High Top')
        self.failUnless(IKeyword('1 intercept') in oHighTop.keywords)
        oGypsies = IAbstractCard('Gypsies')
        self.failUnless(IKeyword('1 stealth') in oGypsies.keywords)
        oRebekka = IAbstractCard('Rebekka, Chantry Elder of Munich')
        self.failUnless(IKeyword('1 stealth') in oRebekka.keywords)
        oProtracted = IAbstractCard('Protracted Investment')
        self.failUnless(IKeyword('investment') in oProtracted.keywords)
        oBravo = IAbstractCard('Bravo')
        self.failUnless(IKeyword('archetype') in oBravo.keywords)
        oDramatic = IAbstractCard('Dramatic Upheaval')
        self.failUnless(IKeyword('not for legal play') in oDramatic.keywords)
        oMotivated = IAbstractCard('Motivated by Gehenna')
        self.failUnless(IKeyword('not for legal play') in oMotivated.keywords)

        # Test adding extra expansions works

        oAnarchRailroad = IAbstractCard("Anarch Railroad")
        self.assertEqual(oAnarchRailroad.canonicalName, u"anarch railroad")
        self.assertEqual(oAnarchRailroad.cost, 2)
        self.assertEqual(oAnarchRailroad.costtype, 'pool')
        self.failUnless(oAnarchRailroad.text.startswith(u"Master: unique"))
        oAnarchs = IExpansion('Anarchs')
        oAA = IExpansion('Anarchs and Alastors Storyline')

        self.assertTrue(oAnarchs in [oP.expansion for oP in
            oAnarchRailroad.rarity])
        self.assertTrue(oAA in [oP.expansion for oP in oAnarchRailroad.rarity])

        oAnarchRevolt = IAbstractCard("Anarch Revolt")
        self.assertEqual(oAnarchRevolt.canonicalName, u"anarch revolt")
        self.assertEqual(oAnarchRevolt.cost, None)
        self.failUnless(oAnarchRevolt.text.startswith(u"Master."))

        self.assertTrue(oAnarchs in [oP.expansion for oP in
            oAnarchRevolt.rarity])
        self.assertTrue(oAA in [oP.expansion for oP in oAnarchRevolt.rarity])
        self.assertTrue(oJyhad in [oP.expansion for oP in
            oAnarchRevolt.rarity])

        oHtH = IAbstractCard("Hide the Heart")
        self.assertEqual(oHtH.canonicalName, u"hide the heart")
        self.assertEqual(len(oHtH.discipline), 2)
        self.failUnless(oAusSup in oHtH.discipline)
        self.failUnless(oValSup in oHtH.discipline)
        self.assertEqual(len(oHtH.cardtype), 1)
        self.failUnless(ICardType("Reaction") in oHtH.cardtype)
        self.failUnless(oHtH.text.startswith('[aus] Reduce'))

        oSmite = IAbstractCard("Smite")
        self.assertEqual(oSmite.canonicalName, u"smite")
        self.assertEqual(oSmite.cost, 3)
        self.assertEqual(oSmite.costtype, "conviction")
        self.failUnless(oSmite.text.startswith('{Strike:}'))


if __name__ == "__main__":
    unittest.main()



# TestData.py
# -*- coding: utf8 -*-
# vim:fileencoding=cp1252 ai ts=4 sts=4 et sw=4
# Based on the card data published by WhiteWolf at:
#  * http://www.white-wolf.com/vtes/index.php?line=cardlist
#  * http://www.white-wolf.com/vtes/index.php?line=rulings

"""Data used for test cases"""
# pylint: disable-msg=C0103
# don't care about naming conventions here

# Note - Yvette & Aire of Elation are deliberately duplicated, so we ensure
# that duplicates in the cardlist are handled properly
TEST_CARD_LIST = """
Name: .44 Magnum
[Jyhad:C, VTES:C, Sabbat:C, SW:PB, CE:PTo3, LoB:PO3]
Cardtype: Equipment
Cost: 2 pool
Weapon, gun.
2R damage each strike, with an optional maneuver each combat.
Artist: Né Né Thomas; Greg Simanson

Name: AK-47
[LotN:R]
Cardtype: Equipment
Cost: 5 pool
Weapon. Gun.
2R damage each strike, with an optional maneuver ={each combat}=. When bearer strikes with this gun, he or she gets an optional additional strike this round, only usable to strike with this gun.
Artist: Franz Vohwinkel

Name: Aabbt Kindred
[FN:U2]
Cardtype: Vampire
Clan: Follower of Set
Group: 2
Capacity: 4
Discipline: for pre ser
Independent: Aabbt Kindred cannot perform (D) actions unless Nefertiti is ready. Aabbt Kindred can prevent 1 damage each combat. Aabbt Kindred are not unique and do not contest.
Artist: Lawrence Snelly

Name: Aaron Bathurst
[Third:V]
Cardtype: Vampire
Clan: Nosferatu antitribu
Group: 4
Capacity: 4
Discipline: for obf pot
Sabbat.
Artist: Rik Martin

Name: Aaron Duggan, Cameron's Toady
[Sabbat:V, SW:U]
Cardtype: Vampire
Clan: Lasombra
Group: 2
Capacity: 2
Discipline: obt
Sabbat
Artist: Eric LaCombe

Name: Aaron's Feeding Razor
[Jyhad:R, VTES:R, CE:R, KoT:R]
Cardtype: Equipment
Cost: 1 pool
Unique equipment.
When this vampire successfully hunts, he or she gains 1 additional blood.
Artist: Thomas Nairb; Christopher Rush

Name: Abandoning the Flesh
[CE:R, Third:R]
Cardtype: Reaction/Combat
Discipline: Dementation
Only usable by a vampire being burned. Usable by a vampire in torpor.
[dem] Remove this vampire from the game instead (diablerie, if any, is still successful), and put this card into play. You may not play this card if you already have an Abandoning the Flesh in play. You may tap this card when a vampire with Dementation is bleeding to give that vampire +1 bleed for the current action.
Artist: Steve Ellis

Name: Abbot
[Third:U]
Cardtype: Action
+1 stealth action. Requires a Sabbat vampire.
Put this card on this acting Sabbat vampire and untap him or her. This Sabbat vampire gets +1 intercept against (D) actions directed at his or her controller. A vampire may have only one Abbot.
Artist: John Bridges

Name: Abd al-Rashid
[AH:V3, FN:PA]
Cardtype: Vampire
Clan: Assamite
Group: 2
Capacity: 5
Discipline: obf CEL QUI
Independent: (Blood Cursed)
Artist: Tom Wänerstrand

Name: Abdelsobek
[LotN:U]
Cardtype: Vampire
Clan: Follower of Set
Group: 5
Capacity: 5
Discipline: for nec obf pre ser
Independent: Abdelsobek can untap a vampire or mummy you control as a +1 stealth action.
Artist: Ken Meyer, Jr.

Name: Abebe
[LoB:U]
Cardtype: Vampire
Clan: Samedi
Group: 4
Capacity: 4
Discipline: nec obf thn
Independent.
Artist: James Stowe

Name: Abjure
[NoR:R]
Cardtype: Power
Virtue: Redemption
[COMBAT] Tap this imbued before range is determined to end a combat between a monster and a mortal. If the mortal is a minion other than this imbued, you may move a conviction to this imbued from your hand or ash heap.
Artist: Brian LeBlanc

Name: Ablative Skin
[Sabbat:R, SW:R, Third:R]
Cardtype: Action
Discipline: Fortitude
+1 stealth action.
[for] Put this card on the acting vampire and put 3 ablative counters on this card. While in combat, this vampire may remove any number of ablative counters from this card to prevent that amount of non-aggravated damage. Burn this card when it has no more ablative counters.
[FOR] As above, but this vampire may also prevent aggravated damage in combat in this way.
Artist: Richard Thomas

Name: Abombwe [abo]
[LoB:C/PA]
Cardtype: Master
Capacity: +1
Master: Discipline. Trifle.
Put this card on a Laibon or on a vampire with Protean [pro]. This vampire gains one level of Abombwe [abo]. Capacity increases by 1: the vampire is one generation older. Cannot be placed on a vampire with superior Abombwe.
Artist: Ken Meyer, Jr.

Name: Aeron
[Gehenna:U]
Cardtype: Vampire
Clan: Nosferatu antitribu
Group: 3
Capacity: 9
Discipline: aus pro ANI OBF POT
Sabbat Archbishop of London: Minions opposing Aeron in combat take an additional point of damage during strike resolution if the range is close. Once each combat, Aeron may burn a blood for a press.
Artist: Ken Meyer, Jr.

Name: Agent of Power
[LotN:C, HttB:PSam4]
Cardtype: Master
Master: Discipline. Trifle. Unique.
Put this card on a vampire you control and choose a Discipline. This vampire gains 1 level of that Discipline. Burn this card during your discard phase.
Artist: Jeff Holt

Name: Aire of Elation
[DS:C3, FN:PS3, CE:C/PTo3, Anarchs:PAB, KMW:PAn2]
Cardtype: Action Modifier
Cost: 1 blood
Discipline: Presence
You cannot play another action modifier to further increase the bleed for this action.
[pre] +1 bleed; +2 bleed if the acting vampire is Toreador.
[PRE] +2 bleed; +3 bleed if the acting vampire is Toreador.
Artist: Greg Simanson

Name: Aire of Elation
[DS:C3, FN:PS3, CE:C/PTo3, Anarchs:PAB, KMW:PAn2]
Cardtype: Action Modifier
Cost: 1 blood
Discipline: Presence
You cannot play another action modifier to further increase the bleed for this action.
[pre] +1 bleed; +2 bleed if the acting vampire is Toreador.
[PRE] +2 bleed; +3 bleed if the acting vampire is Toreador.
Artist: Greg Simanson

Name: Akram
[AH:V3, CE:PB]
Cardtype: Vampire
Clan: Brujah
Group: 2
Capacity: 8
Discipline: pot pre CEL QUI
Camarilla primogen: Once each turn after completing combat, if Akram and the opposing minion are still ready, Akram may burn 1 blood to begin another combat with the opposing minion.
Artist: Terese Nielsen

Name: Alan Sovereign
[CE:V, BSC:X]
Cardtype: Vampire
Clan: Ventrue
Group: 3
Capacity: 6
Discipline: for pre AUS DOM
Camarilla: When you play an investment card, add an additional counter to it from the blood bank.
Artist: Steve Prescott

Name: Alan Sovereign
[Promo-20051001]
Cardtype: Vampire
Clan: Ventrue
Level: Advanced
Group: 3
Capacity: 6
Discipline: for pre AUS DOM
Advanced, Camarilla: While Alan is ready, you may pay some or all of the pool cost of equipping from any investment cards you control.
[MERGED] During your master phase, if Alan is ready, you may move a counter from any investment card to your pool.
Artist: Leif Jones

Name: Alexandra
[DS:V, CE:PTo]
Cardtype: Vampire
Clan: Toreador
Group: 2
Capacity: 11
Discipline: dom ANI AUS CEL PRE
Camarilla Inner Circle: Once during your turn, you may tap or untap another ready Toreador. +2 bleed.
Artist: Lawrence Snelly

Name: Alfred Benezri
[CE:V, BSC:X]
Cardtype: Vampire
Clan: Pander
Group: 3
Capacity: 6
Discipline: aus dom PRE THA
Sabbat bishop: Alfred gets -1 strength in combat with an ally.
Artist: Quinton Hoover

Name: Ambrogino Giovanni
[FN:U2]
Cardtype: Vampire
Clan: Giovanni
Group: 2
Capacity: 9
Discipline: aus DOM NEC POT THA
Independent: Ambrogino has 1 vote. +1 bleed.
Artist: Christopher Shy

Name: Amisa
[AH:V3, FN:PS]
Cardtype: Vampire
Clan: Follower of Set
Group: 2
Capacity: 8
Discipline: pre pro OBF SER
Independent: Amisa has 2 votes. Amisa can tap a vampire with a capacity above 7 as a (D) action.
Artist: Pete Venters

Name: Anastasz di Zagreb
[CE:V, KMW:PAl, BSC:X]
Cardtype: Vampire
Clan: Tremere
Group: 3
Capacity: 8
Discipline: ani cel dom AUS THA
Camarilla Tremere Justicar: If there are any other justicars ready, Anastasz gets 1 fewer vote from his justicar title. Anastasz may steal 1 blood as a ranged strike.
Artist: Christopher Shy

Name: Angelica, The Canonicus
[Sabbat:V, SW:PL]
Cardtype: Vampire
Clan: Lasombra
Group: 2
Capacity: 10
Discipline: cel obf DOM OBT POT
Sabbat cardinal: Once each ={action that}= Angelica attempts to block, you may burn X master cards from your hand to give her +X intercept.
Artist: John Bolton

Name: The Ankara Citadel, Turkey
[AH:U5, CE:U, KoT:U]
Cardtype: Equipment
Clan: Tremere
Cost: 2 blood
This equipment card represents a unique location and does not count as equipment while in play.
The vampire with this location pays only half of the blood cost for any cards he or she plays (round down).
Artist: Greg Simanson; Brian LeBlanc

Name: Anna "Dictatrix11" Suljic
[NoR:U]
Cardtype: Imbued
Creed: Martyr
Group: 4
Life: 6
Virtue: mar red vis
Anna may move 2 blood from the blood bank to any vampire as a +1 stealth action. During your untap phase, you may look at the top three cards of your library.
Artist: Thomas Manning

Name: Anson
[Jyhad:V, VTES:V, Tenth:A]
Cardtype: Vampire
Clan: Toreador
Group: 1
Capacity: 8
Discipline: aus dom CEL PRE
Camarilla Prince of Seattle: If Anson is ready during your master phase, you get two master phase actions (instead of one).
Artist: Mark Tedin

Name: Bravo
[Gehenna:C]
Cardtype: Master
Master: archetype.
Put this card on a vampire you control. Once per turn, when this vampire successfully performs an action to enter combat with another, he or she gains 1 blood from the blood bank when the combat ends, if he or she is still ready. A vampire can have only one archetype.
Artist: Nilson

Name: Bronwen
[Sabbat:V, SW:PB]
Cardtype: Vampire
Clan: Brujah antitribu
Group: 2
Capacity: 10
Discipline: dom obt CEL POT PRE
Sabbat priscus: Once each combat, Bronwen may dodge as a strike.
Artist: Ken Meyer, Jr.

Name: Cedric
[LoB:U, HttB:PGar2]
Cardtype: Vampire
Clan: Gargoyle
Group: 4
Capacity: 6
Discipline: obf pot vis FOR
Camarilla. Tremere slave: If Cedric successfully blocks a (D) action, he may burn 1 blood when the action ends (after combat, if any) to untap. Flight [FLIGHT].
Artist: David Day

Name: Cesewayo
[LoB:PO2]
Cardtype: Vampire
Clan: Osebo
Group: 4
Capacity: 10
Discipline: ani AUS CEL DOM POT THA
Laibon magaji: Once each action, Cesewayo may burn 1 blood to get +1 intercept.
Artist: Abrar Ajmal

Name: Dramatic Upheaval
[Jyhad:V, VTES:V, CE:U, Anarchs:PAB, BH:PM]
Cardtype: Political Action
Choose another Methuselah. Successful referendum means you switch places with that Methuselah. ={Added to the V:EKN banned list in 2005.}=
Artist: Heather Hudson; Mike Huddleston

Name: Earl "Shaka74" Deams
[NoR:U]
Cardtype: Imbued
Creed: Visionary
Group: 4
Life: 6
Virtue: jud mar vis
Earl gets +1 stealth on actions other than actions to enter combat. During your untap phase, if you control more than two ready imbued, Earl burns 1 conviction [1 CONVICTION].
Artist: David Day

Name: Enkidu, The Noah
[KMW:U/PG]
Cardtype: Vampire
Clan: Gangrel antitribu
Group: 4
Capacity: 11
Discipline: for ANI CEL OBF POT PRO
Sabbat. Red List: Enkidu can enter combat with any minion as a (D) action. If Enkidu successfully performs an action to employ a retainer, he untaps at the end of the turn. He cannot have or use equipment. +2 strength.
Artist: Mark Nelson

Name: Fidus, The Shrunken Beast
[BL:U2]
Cardtype: Vampire
Clan: Gargoyle
Group: 2
Capacity: 4
Discipline: for tha vis
Camarilla Tremere Slave: Fidus gets +1 stealth on undirected actions. -1 strength. Flight [FLIGHT].
Artist: Christopher Shy

Name: Ghoul Retainer
[Jyhad:R2, VTES:R, CE:R2, KoT:U/R]
Cardtype: Retainer
Cost: 2 pool
Ghoul with 2 life. 1 strength.
During the initial strike resolution each round, the Ghoul Retainer inflicts 1 damage or may use a weapon not used by the employing minion (or another retainer) that round (either before or after). This is not a strike, although it does count as "using" the weapon.
Artist: L. A. Williams; Richard Thomas

Name: Gracis Nostinus
[CE:V, BSC:X]
Cardtype: Vampire
Clan: Ventrue
Group: 3
Capacity: 7
Discipline: aus for DOM PRE
Camarilla Primogen: If a younger vampire attempts to block Gracis and fails, tap that vampire at the end of the action.
Artist: Max Shade Fellwalker

Name: Gypsies
[Jyhad:U, VTES:U]
Cardtype: Ally
Clan: Gangrel
Cost: 3 pool
Unique -{mortal}- with 1 life. 1 {strength}, 1 bleed.
Gypsies get +1 stealth on each of their actions.
Artist: Pete Venters

Name: High Top
[BL:R1, LoB:R]
Cardtype: Ally
Clan: Ahrimane
Cost: 4 pool
Unique werewolf with 3 life. 1 strength, 0 bleed.
High Top gets +1 intercept. High Top may enter combat with any minion controlled by another Methuselah as a (D) action. High Top gets an additional strike each round and an optional maneuver once each combat. He may play cards requiring basic Celerity [cel] as a vampire with a capacity of 4. If High Top has less than 3 life during your untap phase, he gains 1 life.
Artist: Mark Nelson

Name: Inez "Nurse216" Villagrande
[NoR:U]
Cardtype: Imbued
Creed: Innocent
Group: 4
Life: 3
Virtue: inn
When Inez enters play, you may search your library (shuffle afterward) or hand for a power that requires innocence and put it on her.
Artist: Jim Pavelec

Name: Kabede Maru
[LotN:U]
Cardtype: Vampire
Clan: Assamite
Group: 5
Capacity: 9
Discipline: abo pot AUS CEL OBF QUI
Laibon magaji: Kabede gets +1 intercept against political actions. (The blood curse does not affect Kabede.)
Artist: Ken Meyer, Jr.

Name: Kemintiri
[KMW:U]
Cardtype: Vampire
Clan: Follower of Set
Level: Advanced
Group: 2
Capacity: 10
Discipline: aus dom OBF PRE SER THA
Advanced, Independent. Red List: +1 stealth.
[MERGED] Kemintiri has 3 votes (titled). She can play {minion} cards that require Camarilla, Ventrue, and/or a justicar title {as if she met that/those requirement(s)}.
Artist: Lawrence Snelly

Name: Lázár Dobrescu
[AH:V3, FN:PR]
Cardtype: Vampire
Clan: Ravnos
Group: 2
Capacity: 3
Discipline: for
Independent: Lázár may move one blood from {an uncontrolled minion} in your prey's uncontrolled region to a vampire in your uncontrolled region as a (D) action.
Artist: Rebecca Guay

Name: Living Manse
[Sabbat:R, SW:R, Third:R]
Cardtype: Equipment
Clan: Tzimisce
Cost: 1 blood
This equipment card represents a location and does not count as an equipment card while it is in play.
The vampire with this location gets +1 bleed. He or she can burn this card before range is determined to end combat. A vampire may have only one Living Manse.
Artist: Mark Tedin

Name: Necromancy [nec]
[DS:C2, FN:PG2, LotN:PG]
Cardtype: Master
Capacity: +1
Master: Discipline.
Put this card on a vampire. This vampire gains 1 level of Necromancy [nec]. Capacity increases by 1: the vampire is one generation older. Cannot be placed on a vampire with superior Necromancy.
Artist: Anson Maddocks; Sam Araya

Name: Ossian
[KMW:R, KoT:R]
Cardtype: Ally
Cost: 3 pool
Unique werewolf with 4 life. 2 strength, 0 bleed. Red List.
Ossian may enter combat with any vampire as a +1 stealth (D) action. In the first round of combat with a vampire who has played a card that requires Auspex [aus] during this action, that vampire cannot use any maneuvers or strikes. Ossian gains 1 life at the end of each round for each blood the opposing vampire used to heal damage or prevent destruction that round.
Artist: Roel Wielinga

Name: Pariah
[KMW:U]
Cardtype: Vampire
Clan: Abomination
Group: 2
Capacity: 6
Discipline: pot pre OBF PRO
Independent: During your master phase, discard a master card or tap Pariah. Pariah cannot take undirected actions other than hunting. He can enter combat with any minion as a (D) action. +1 strength. Scarce. Sterile.
Artist: Richard Thomas

Name: Paris Opera House
[BL:R1, LoB:R]
Cardtype: Master
Clan: Daughter of Cacophony
Cost: 2 pool
Burn Option
Master: unique location.
Tap to give a Daughter of Cacophony you control +1 stealth. Tap this card and a Daughter of Cacophony you control to give any minion +1 stealth.
Artist: William O'Connor

Name: Park Hunting Ground
[DS:C2, FN:PR, LotN:PR]
Cardtype: Master
Clan: Ravnos
Cost: 2 pool
Master: unique location. Hunting ground.
During your untap phase, you may move 1 blood from the blood bank to a ready vampire you control. A vampire can gain blood from only one Hunting Ground card each turn.
Artist: Pete Venters; Sam Araya

Name: The Path of Blood
[AH:C2, FN:PA, LotN:PA2]
Cardtype: Master
Clan: Assamite
Cost: 1 pool
Unique master.
Put this card in play. Cards that require Quietus [qui] {cost Assamites 1 less blood}. Any minion may burn this card as a (D) action; if that minion is a vampire, he or she then takes 1 unpreventable damage when this card is burned.
Artist: Drew Tucker; Jeff Holt

Name: Pier 13, Port of Baltimore
[SW:U/PB, Third:U, KoT:U]
Cardtype: Equipment
Cost: 2 blood
This equipment card represents a unique location and does not count as equipment while in play.
During your influence phase, this minion may equip with a non-location, non-unique equipment card from your hand (requirements and cost apply as normal). This is not an action and cannot be blocked.
Artist: Steve Prescott

Name: Political Hunting Ground
[Sabbat:U, SW:U/PL, HttB:PKia]
Cardtype: Master
Clan: Lasombra
Cost: 2 pool
Master: unique location. Hunting ground.
During your untap phase, you may move 1 blood from the bank to a ready vampire you control. A vampire can gain blood from only one hunting ground card each turn.
Artist: John Scotello; Melissa Uran

Name: Predator's Communion
[LoB:C]
Cardtype: Reaction
Discipline: Abombwe
[abo] [REFLEX] Cancel a frenzy card played on this vampire as it is played.
[abo] +1 intercept. Only usable when a vampire is acting.
[ABO] Only usable by a tapped vampire when a vampire is acting. This reacting vampire untaps.
Artist: David Day

Name: Protracted Investment
[Jyhad:C, VTES:C, CE:PTr]
Cardtype: Master
Cost: 2 pool
Master. Investment.
{Put this card in play and} move 5 blood from the blood bank to this card. You may use a master phase action to move 1 blood from this card to your pool. Burn this card when all blood has been removed.
Artist: Brian Snoddy

Name: Raven Spy
[Jyhad:U, VTES:U, CE:U/PN, Anarchs:PG2, BH:PN3, KMW:PG, Third:PTz3, LotN:PR3, KoT:U]
Cardtype: Retainer
Cost: 1 blood
Discipline: Animalism
Animal with 1 life.
[ani] This minion gets +1 intercept.
[ANI] As above, but the Raven Spy has 2 life.
Artist: Jeff Holt; Dan Frazier

Name: Rebekka, Chantry Elder of Munich
[DS:V, CE:PTr]
Cardtype: Vampire
Clan: Tremere
Group: 2
Capacity: 8
Discipline: pot AUS PRE THA
Camarilla: Rebekka gets +1 stealth on each of her actions. Rebekka gets +1 bleed when bleeding a Methuselah who controls a ready Malkavian.
Artist: Anson Maddocks

Name: Rock Cat
[BL:R1, LoB:R, HttB:PGar]
Cardtype: Ally
Clan: Gargoyle
Cost: 4 pool
Gargoyle creature with 4 life. 3 strength, 0 bleed.
Rock Cat may enter combat with a ready minion as a (D) action. Opposing vampires with capacity 3 or less cannot strike in the first round. Rock Cat gets an optional press each combat. Rock Cat may play cards requiring basic Potence [pot] as a 3-capacity vampire.
Artist: Jeff Holt

Name: Scapelli, The Family "Mechanic"
[DS:U2]
Cardtype: Ally
Clan: Giovanni
Cost: 3 pool
Unique -{mortal}- with 3 life. {0 strength}, 1 bleed.
{Scapelli may strike for 2R damage.} Once each combat, Scapelli may press to continue combat.
Artist: Richard Thomas

Name: Shade
[Sabbat:U, SW:PL]
Cardtype: Retainer
Cost: 1 blood
Discipline: Obtenebration
-{Demon}- with 2 life.
[obt] When the minion with this retainer is in combat, the opposing minion takes 1 damage during strike resolution {if the range is close}.
[OBT] As above, but Shade has 3 life.
Artist: Stuart Beel

Name: Sha-Ennu
[Third:V]
Cardtype: Vampire
Clan: Tzimisce
Group: 4
Capacity: 11
Discipline: obf tha ANI AUS CHI VIC
Sabbat regent: Vampires with capacity less than 4 cannot block Sha-Ennu. +2 bleed.
Artist: Richard Thomas

Name: The Siamese
[BL:U2]
Cardtype: Vampire
Clan: Ahrimane
Group: 2
Capacity: 7
Discipline: ani pro PRE SPI
Sabbat: +1 bleed. Sterile.
Artist: Lawrence Snelly

Name: The Slaughterhouse
[BL:C1, LoB:C]
Cardtype: Master
Clan: Harbinger of Skulls
Cost: 1 pool
Burn Option
Master: location.
Tap to burn two cards from the top of your prey's library.
Artist: William O'Connor

Name: Two Wrongs
[Promo-20081119]
Cardtype: Master
Master: out-of-turn. Trifle.
Play when a minion controlled by a Methuselah other than your predator is bleeding you -{after blocks are declined}-. That minion is now bleeding his or her prey. The next card that would change the target of this bleed is canceled as it is played.
Artist: Leif Jones

Name: Vox Domini
[BH:R, LoB:PA]
Cardtype: Master
Cost: 1 pool
Master: out-of-turn.
Only usable during the referendum of a political action. Not usable on a referendum that is automatically passing. The referendum fails. Each Methuselah may play only one Vox Domini each game.
Artist: Christopher Shy

Name: Yvette, The Hopeless
[CE:V/PTo, BSC:X]
Cardtype: Vampire
Clan: Toreador
Group: 3
Capacity: 3
Discipline: aus cel
Camarilla.
Artist: Leif Jones

Name: Hide the Heart
[HttB:C/PSal2]
Cardtype: Reaction
Discipline: Valeren / Auspex

[aus] Reduce a bleed against you by 1.
[val] The action ends (unsuccessfully). The acting minion may burn 1 blood to cancel this card as it is played. Only one Hide the Heart may be played at [val] each action.
[VAL] Reduce a bleed against you by 2, or tap to reduce a bleed against any Methuselah by 2.
Artist: Kari Christensen

Name: Anarch Railroad
[Anarchs:R2]
Cardtype: Master
Cost: 2 pool
Master: unique location.
Tap to give an anarch +1 stealth for the current action.
Artist: Joel Biske

Name: Anarch Revolt
[Jyhad:U, VTES:U, CE:U, Anarchs:PAB2, KMW:PAn, Third:U]
Cardtype: Master
Master.
Put this card in play. A Methuselah who does not control a ready anarch burns 1 pool during his or her untap phase. Any vampire can call a referendum to burn this card as a +1 stealth political action.
Artist: Pete Venters; Steve Prescott

Name: Anarch Railroad
[Anarchs and Alastors Storyline]

Name: Anarch Revolt
[Anarchs and Alastors Storyline]

Name: Smite
[NoR:R]
Cardtype: Combat
Cost: 3 Conviction
Virtue: Vengeance
{Strike:} strength+1 aggravated ranged damage. Even if the strike is dodged, burn any electronic equipment (e.g., IR Goggles, Laptop Computer, or Phased Motion Detector) on either combatant.
Artist: Heather Kreiter

Name: Motivated by Gehenna
[Eden's Legacy Storyline:Storyline]
Cardtype: Master
Master.
Put this card in play, If you control the Edge, any vampire you control may enter combat with a ready minion controlled by another Methuselah as a (D) action that costs 1 blood.  {NOT FOR LEGAL PLAY}
"""

TEST_RULINGS = """
<html>
<head><title>Sutekh Test Rulings</title></head>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<body id="page_bg">
    <div id="pillmenu"><ul class="menu"><li class="item5"><a href="http://www.vekn.net/"><span>Home</span></a></li><li class="item2"><a href="/index.php/forum/index"><span>Forum</span></a></li><li class="item117"><a href="/index.php/event-calendar"><span>Event Calendar</span></a></li></ul><form action="http://www.vekn.net/index.php/component/comprofiler/login" method="post" id="mod_loginform" class="cbLoginForm" style="margin:0px;">

<tr>
<td valign="top">

<p><b>AK-47:</b></p>
<ul>
<li>The AK-47 provides the bearer one optional maneuver "each combat". <a target="_top" href="http://groups-beta.google.com/groups?selm=46FCEF4C.6050100%40white-wolf.com">[LSJ 20070928]</a></li>
</ul>

<p><b>Ablative Skin:</b></p>
<ul>
<li>Cannot be used to prevent damage that cannot be prevented by cards that require Fortitude (e.g., Blood Rage and Blood Fury). <a target="_top" href="http://groups-beta.google.com/groups?selm=36C98715.81740A4B%40wizards.com">[LSJ 19990216]</a></li>
</ul>

<p><b>Lazar Dobrescu:</b></p>
<ul>
<li>Cannot use his special ability if his controller has no vampires in her uncontrolled region. <a target="_top" href="http://groups-beta.google.com/groups?selm=36C82B01.3705D2F0%40wizards.com">[LSJ 19990215]</a></li>
</ul>

</body></html>
"""



# FilterBox.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Filter Box abstraction used by the editor.
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# Split out from FilterEditor 2010 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""FilterBox model of the filters. Mainly used in filter editor"""

from sutekh.core.FilterParser import FilterNode, BinOpNode, NotOpNode, \
        FilterPartNode, escape


class FilterBoxModel(list):
    """Converts a filter AST into a simple nest box model.

       Each box either ANDs or ORs all of its contained filters.
       Each contained filter may optionally be negated.
       Values are retained for each contained filter.
       """

    AND, OR = 'and', 'or'

    # pylint: disable-msg=W0231
    # no point to calling list's __init__
    def __init__(self, oAST, sFilterType, oVarNameMaker=None):
        """Initialise a filter box from an AST filter representation."""
        if oVarNameMaker is None:
            oVarNameMaker = VariableNameGenerator()
            bVarMakerNeedsInit = True
        else:
            bVarMakerNeedsInit = False

        self.sFilterType = sFilterType
        self.bNegate = False
        self.bDisabled = False
        self.oVarNameMaker = oVarNameMaker

        if type(oAST) is FilterNode:
            oAST = oAST.oExpression

        if type(oAST) is BinOpNode:
            self.sBoxType = oAST.oOp
            self._init_binop(oAST)
        elif type(oAST) is NotOpNode and \
                type(oAST.oSubExpression) is BinOpNode:
            self.bNegate = True
            self.sBoxType = oAST.oSubExpression.oOp
            self._init_binop(oAST.oSubExpression)
        elif type(oAST) is FilterPartNode or \
                type(oAST) is NotOpNode and \
                type(oAST.oSubExpression) is FilterPartNode:
            self.sBoxType = self.AND
            self.append(FilterBoxItem(oAST, self.oVarNameMaker))
        elif oAST is None:
            # support for completely empty boxes
            self.sBoxType = self.AND
        else:
            raise ValueError("FilterBoxModel cannot represent AST %s of"
                    " type %s" % (oAST, type(oAST)))

        assert self.sBoxType in (self.AND, self.OR)

        if bVarMakerNeedsInit:
            self.oVarNameMaker.update(self.get_variable_names())

    def __eq__(self, oOther):
        """Non-list equality check so empty boxes aren't equal"""
        return self is oOther

    def _init_binop(self, oBinOp):
        """Create the correct box entries for a BinOpNode in the AST."""
        for oChild in (oBinOp.oLeft, oBinOp.oRight):
            if type(oChild) is BinOpNode and oChild.oOp == oBinOp.oOp:
                self._init_binop(oChild)
            elif type(oChild) is BinOpNode:
                self.append(FilterBoxModel(oChild, self.sFilterType,
                    self.oVarNameMaker))
            elif type(oChild) is NotOpNode and \
                    type(oChild.oSubExpression) is BinOpNode:
                self.append(FilterBoxModel(oChild, self.sFilterType,
                    self.oVarNameMaker))
            elif type(oChild) in (NotOpNode, FilterPartNode):
                self.append(FilterBoxItem(oChild, self.oVarNameMaker))
            else:
                raise ValueError("FilterBoxModel encountered unsupported AST"
                        " node type %s (%s) while examing BinOpNode tree." %
                        (type(oChild), oChild))

    def set_boxtype(self, sBoxType, bNegate=False):
        """Set the type for this filter box"""
        self.sBoxType = sBoxType
        self.bNegate = bNegate

    def get_variable_names(self):
        """Get the variable names of the children in this box."""
        oNames = set()
        for oChild in self:
            oNames.update(oChild.get_variable_names())
        return oNames

    def get_ast(self):
        """Return an AST representation of the filter."""
        if self.bDisabled:
            return None

        aChildASTs = [oChild.get_ast() for oChild in self]
        aChildASTs = [oChild for oChild in aChildASTs if oChild is not None]

        if len(aChildASTs) == 0:
            oAST = None
        elif len(aChildASTs) == 1:
            oAST = aChildASTs[0]
        else:
            oAST = BinOpNode(aChildASTs[0], self.sBoxType, aChildASTs[1])
            for oChild in aChildASTs[2:]:
                oAST = BinOpNode(oAST, self.sBoxType, oChild)

        if self.bNegate and oAST is not None:
            oAST = NotOpNode(oAST)

        return oAST

    def get_ast_with_values(self):
        """Return AST with values filled in"""
        oNewAST = self.get_ast()
        if oNewAST:
            dValues = self.get_current_values()
            aNewValues = oNewAST.get_values()
            for oValue in aNewValues:
                if oValue.is_entry() or oValue.is_list() or oValue.is_tuple():
                    oValue.oNode.set_values(dValues[oValue.oNode.get_name()])

        return oNewAST

    def get_text(self):
        """Return a text representation of the filter."""
        aChildTexts = [oChild.get_text() for oChild in self]
        aChildTexts = [sChild for sChild in aChildTexts if sChild]

        if len(aChildTexts) == 0:
            sText = ''
        elif len(aChildTexts) == 1:
            sText = aChildTexts[0]
        else:
            sText = "(" + (") %s (" % self.sBoxType).join(aChildTexts) + ")"

        if self.bNegate and sText:
            sText = "NOT (%s)" % (sText,)

        return sText

    def get_current_values(self):
        """Return a dictionary of name -> value mappings for the variables
           present in the filter editing gui.
           """
        dVars = {}
        for oChild in self:
            dVars.update(oChild.get_current_values())
        return dVars

    def add_child_box(self, sChildBoxType):
        """Add a child box to this box."""
        assert sChildBoxType in (self.AND, self.OR)
        oChildBox = FilterBoxModel(None, self.sFilterType, self.oVarNameMaker)
        oChildBox.sBoxType = sChildBoxType
        self.append(oChildBox)
        return oChildBox

    def add_child_item(self, sChildTypeKeyword):
        """Add a filter item to this box."""
        sVarName = self.oVarNameMaker.generate_name()
        oAST = FilterPartNode(sChildTypeKeyword, None, sVarName)
        oChildItem = FilterBoxItem(oAST)
        self.append(oChildItem)
        return oChildItem

    def remove_child(self, oChild):
        """Remove a filter item or box from this box."""
        self.remove(oChild)

    def is_in_model(self, oChild):
        """Check if oChild is included in this box model, either directly
           or in a child box model."""
        if oChild in self:
            return True
        for oObj in self:
            if isinstance(oObj, FilterBoxModel) and oObj.is_in_model(oChild):
                return True
        return False


class FilterBoxItem(object):
    """A item in the filter editor.

       This represents either a single FilterPart or a single
       NOT(FilterPart) expression in the AST.
       """
    # pylint: disable-msg=R0902
    # We track a lot of state, so several instance attributes
    NONE, ENTRY, LIST, LIST_FROM = range(4)

    def __init__(self, oAST, oVarNameMaker=None):
        if type(oAST) is NotOpNode:
            self.bNegated = True
            oAST = oAST.oSubExpression
        else:
            self.bNegated = False

        assert type(oAST) is FilterPartNode

        self.bDisabled = False

        self.sFilterName = oAST.sFilterName
        self.sFilterDesc = oAST.get_description()
        self.sVariableName = oAST.sVariableName
        if not self.sVariableName and oVarNameMaker is not None:
            self.sVariableName = oVarNameMaker.generate_name()
        elif self.sVariableName and oVarNameMaker is not None:
            if self.sVariableName in oVarNameMaker:
                self.sVariableName = oVarNameMaker.generate_name()
            else:
                oVarNameMaker.update([self.sVariableName])
        self.iValueType = None

        self.sLabel, self.aValues = None, None
        self.aCurValues = []
        # process values
        self._set_value_type(oAST)
        assert self.sLabel is not None
        assert self.iValueType is not None

        self._set_values(oAST)

    def _set_value_type(self, oAST):
        """Set the value type and label for the filter"""
        for oValue in oAST.get_values():
            if oValue.is_value():
                assert self.sLabel is None
                self.sLabel = oValue.oValue
            elif oValue.is_list():
                assert self.iValueType is None
                self.aValues = oValue.oValue
                self.iValueType = self.LIST
            elif oValue.is_tuple():
                assert self.iValueType is None
                self.aValues = oValue.oValue
                self.iValueType = self.LIST_FROM
                self.aCurValues = [None, None]
            elif oValue.is_entry():
                assert self.iValueType is None
                self.iValueType = self.ENTRY
            elif oValue.is_None():
                assert self.iValueType is None
                self.iValueType = self.NONE

    def _set_values(self, oAST):
        """Initialise the values of the filter item"""
        aFilterValues = oAST.aFilterValues
        if aFilterValues:
            if self.iValueType == self.LIST_FROM:
                assert len(aFilterValues.get_values()) == 2
                oLeft, oRight = aFilterValues.get_values()
                aFrom, aValues = [], []
                for oNode in oLeft:
                    if oNode.oValue != ',':
                        aValues.append(oNode.oValue)
                if '-1' in aValues:
                    aValues = None  # Sentinal case
                for oNode in oRight:
                    if oNode.oValue != ',':
                        aFrom.append(oNode.oValue)
                if "" in aFrom:
                    aFrom = None  # Sentinal case
                self.aCurValues = [aValues, aFrom]
            elif self.iValueType == self.ENTRY:
                assert len(aFilterValues.get_values()) == 1
                self.aCurValues = [aFilterValues.get_values()[0].oValue]
            else:
                # List filter
                for oNode in aFilterValues.get_values():
                    if oNode.oValue != ',':
                        self.aCurValues.append(oNode.oValue)

    def get_variable_names(self):
        """Get the variable name for this filter"""
        return set([self.sVariableName])

    def get_current_values(self):
        """Get the current values set for this filter.

           We escape the values to ensure we handle quotes and so forth
           sanely"""
        dVars = {}
        # Flag as empty by default
        dVars[self.sVariableName] = None
        if self.aCurValues:
            if self.iValueType == self.LIST:
                dVars[self.sVariableName] = ['"%s"' % escape(sValue)
                        for sValue in self.aCurValues]
            elif self.iValueType == self.LIST_FROM:
                aValues, aFrom = self.aCurValues
                if aFrom:
                    aFrom = ['"%s"' % escape(x) for x in aFrom]
                    if aValues:
                        aValues = ['"%s"' % escape(x) for x in aValues]
                        dVars[self.sVariableName] = [aValues, aFrom]
            elif self.iValueType == self.ENTRY:
                dVars[self.sVariableName] = ['"%s"' %
                        escape(self.aCurValues[0])]
        return dVars

    def get_ast(self):
        """Return an AST representation of the filter."""
        if (not self.aCurValues and self.iValueType != self.NONE) \
                or self.bDisabled:
            return None
        oAST = FilterPartNode(self.sFilterName, None,
                self.sVariableName)
        if self.bNegated:
            oAST = NotOpNode(oAST)
        return oAST

    def get_text(self):
        """Return a text representation of the filter."""
        if self.iValueType == self.NONE:
            sText = self.sFilterName
        elif self.aCurValues:
            sValues = None
            sText = "%s in %s" % (self.sFilterName, self.sVariableName)
            if self.iValueType == self.LIST:
                sValues = ",".join(['"%s"' % escape(sValue) for sValue in
                        self.aCurValues])
            elif self.iValueType == self.LIST_FROM:
                aValues, aFrom = self.aCurValues
                sFromValues = '"-1"'
                sFrom = '""'  # Sentinals
                if aValues:
                    sFromValues = ",".join(['"%s"' % escape(x)
                        for x in aValues])
                if aFrom:
                    sFrom = ",".join(['"%s"' % escape(x) for x in aFrom])
                if aFrom or aValues:
                    sValues = "%s FROM %s" % (sFromValues, sFrom)
            elif self.iValueType == self.ENTRY:
                sValues = '"%s"' % escape(self.aCurValues[0])
            if sValues:
                sText = '%s in %s' % (self.sFilterName, sValues)
        else:
            sText = "%s in %s" % (self.sFilterName, self.sVariableName)
        if self.bNegated:
            return "NOT (%s)" % sText
        return sText


class VariableNameGenerator(set):
    """Generate a unique name for a variable in a filter."""
    KEYFORM = "$var%s"

    def __init__(self):
        super(VariableNameGenerator, self).__init__()
        self.__iNum = 0

    def generate_name(self):
        """Generate the next variable in the list."""
        sName = self.KEYFORM % self.__iNum
        while sName in self:
            self.__iNum += 1
            sName = self.KEYFORM % self.__iNum
        self.add(sName)
        return sName

# Useful constants for dealing with the filter boxes

BOXTYPE = {  # description -> (AND or OR, bNegate)
    'All of ...': (FilterBoxModel.AND, False),
    'Any of ...': (FilterBoxModel.OR, False),
    'Not all of ...': (FilterBoxModel.AND, True),
    'None of ...': (FilterBoxModel.OR, True),
}

BOXTYPE_ORDER = [  # order types should appear in combo
    'All of ...', 'Any of ...', 'Not all of ...', 'None of ...',
]



# test_ELDBInventoryParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test reading a card set from an ELDB inventory file"""

import unittest
from sutekh.tests.TestCore import SutekhTest
from sutekh.io.ELDBInventoryParser import ELDBInventoryParser

# FELDB creates 0,0 entries, so they need to be in the test case
ELDB_INV_EXAMPLE_1 = """
"ELDB - Inventory"
"Test Vamp 1",2,0,"","Crypt"
"Lazar Dobrescu",1,0,"","Crypt"
"Test Vamp 2",0,0,"","Crypt"
"Test Card 1",2,0,"","Library"
"Test Card 2",4,0,"","Library"
"Test Card 3",12,0,"","Library"
"Test Card 4",1,0,"","Library"
"Test Card 5",0,0,"","Library"
"""


# Needs to be a SutekhTestCase so the name mapping cache test works
class TestELDBInventoryParser(SutekhTest):
    """class for the ELDB inventory reading test"""

    def test_basic(self):
        """Run the input test."""
        oHolder = self._make_holder_from_string(ELDBInventoryParser(),
                ELDB_INV_EXAMPLE_1)

        self.assertEqual(oHolder.name, '')  # DummyHolder default

        aCards = oHolder.get_cards()

        self.assertEqual(len(aCards), 6)
        self.failUnless(("Test Vamp 1", 2) in aCards)
        self.failUnless((u"L\xe1z\xe1r Dobrescu", 1) in aCards)
        self.failUnless(("Test Card 1", 2) in aCards)
        self.failUnless(("Test Card 2", 4) in aCards)
        self.failUnless(("Test Card 3", 12) in aCards)
        self.failUnless(("Test Card 4", 1) in aCards)
        # We don't need to test that the 0,0 entries have been skipped, as
        # that's covered by the len test

if __name__ == "__main__":
    unittest.main()



# test_SLDeckParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2009 Simon Cross <hodgestar+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test reading a card set from a Secret Library deck file"""

import unittest
import StringIO
from sutekh.tests.TestCore import SutekhTest, DummyHolder
from sutekh.io.SLDeckParser import SLDeckParser


class TestSLDeckParser(SutekhTest):
    """Class for the Secret Library deck parser tests"""

    sTestText1 = """
        ***SL***TITLE***
        NAME OF THE DECK
        ***SL***AUTHOR***
        CREATOR OF THE DECK
        ***SL***CREATED***
        YYYY-MM-DD
        ***SL***DESCRIPTION***
        DESCRIPTION OF THE DECK
        MAY SPAN ON MULTIPLE LINES
        ***SL***CRYPT***
        2 Count Germaine
        2 Count Germaine (Adv)
        2 Fran\xc3\xa7ois Warden Loehr
        ***SL***LIBRARY***
        10 Cloak the Gathering
        2 Coven, The
        2 Carlton Van Wyk (Hunter)
        ***SL***ENDDECK***
        """

    def test_basic(self):
        """Run the Secret Library deck input test."""
        oHolder = DummyHolder()
        fIn = StringIO.StringIO(self.sTestText1)
        oParser = SLDeckParser()
        oParser.parse(fIn, oHolder)

        aCards = oHolder.get_cards()

        self.assertEqual(len(aCards), 6)
        self.failUnless(("Count Germaine", 2) in aCards)
        self.failUnless(("Count Germaine (Advanced)", 2) in aCards)
        self.failUnless(("Fran\xc3\xa7ois Warden Loehr", 2) in aCards)
        self.failUnless(("Cloak the Gathering", 10) in aCards)
        self.failUnless(("The Coven", 2) in aCards)
        self.failUnless(("Carlton Van Wyk (Hunter)", 2) in aCards)
        self.assertEqual(oHolder.name, "NAME OF THE DECK")
        self.assertEqual(oHolder.author, "CREATOR OF THE DECK")
        self.assertEqual(oHolder.comment, "\n".join([
            "Created on YYYY-MM-DD",
            "DESCRIPTION OF THE DECK",
            "MAY SPAN ON MULTIPLE LINES",
        ]))

if __name__ == "__main__":
    unittest.main()



# WwFile.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Tools for dealing with unopened files that follow the White Wolf encoding
   conventions."""

import codecs
import urllib2
import logging

WW_CARDLIST_URL = "http://www.vekn.net/images/stories/downloads/cardlist.txt"

EXTRA_CARD_URL = "http://bitbucket.org/hodgestar/sutekh-extras/" \
        "raw/tip/extra_list.txt"

# pylint: disable-msg=C0103
# We accept the overly long name here
WW_CARDLIST_PER_LETTER_URLS = [
        "http://www.white-wolf.com/vtes/index.php?line=cardlist__",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_A",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_B",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_C",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_D",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_E",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_F",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_G",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_H",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_I",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_J",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_K",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_L",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_M",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_N",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_O",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_P",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_Q",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_R",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_S",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_T",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_U",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_V",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_W",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_X",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_Y",
        "http://www.white-wolf.com/vtes/index.php?line=cardlist_Z",
        ]

# pylint: enable-msg=C0103

WW_OLD_RULINGS_URL = "http://www.white-wolf.com/vtes/index.php?line=rulings"

WW_RULINGS_URL = [
        # We skip the general rulings, since we only want card rulings
        #"http://www.vekn.net/index.php/all-rulings/general-rulings",
        "http://www.vekn.net/index.php/all-rulings/card-rulings-a-to-f",
        "http://www.vekn.net/index.php/all-rulings/card-rulings-g-to-o",
        "http://www.vekn.net/index.php/all-rulings/card-rulings-p-to-z",
        ]


def guess_encoding(sData, sFile):
    """Try to determine the correct encoding from the data"""
    # Try utf8 first, then iso8859-1, which covers what we've
    # seen so far. We may also need one of the cp codecs, but
    # it currently seems not.
    aEncodings = ['utf8', 'iso8859-1']
    for sEnc in aEncodings:
        try:
            logging.info('Trying %s for %s', sEnc, sFile)
            sData.decode(sEnc)
            return sEnc
        except UnicodeDecodeError:
            # We move onto the next one
            pass
    # We've fallen off the back of the loop, so we're in trouble
    # and there're no good choices left, so we raise a RuntimeError
    raise RuntimeError('Unable to indentify correct encoding for %s' % sFile)


class WwFile(object):
    """WwFile is a convenience class which has an .open(..) method which
       returns a file-like object with the encoding set correctly.
       """

    # pylint: disable-msg=C0103
    # we accept sfFile here
    def __init__(self, sfFile, bUrl=False, bFileObj=False):
        self.sfFile = sfFile
        self.bUrl = bUrl
        self.bFileObj = bFileObj

        if bUrl and bFileObj:
            raise ValueError("WwFile cannot be both a URL and a fileobject")

    # pylint: enable-msg=C0103

    def open(self):
        """Return a file object for the file.

           This expects a utf8 encoded file (possibly with BOM) and
           returns a file object producing utf8 strings."""
        if self.bFileObj:
            return self.sfFile
        elif self.bUrl:
            oFile = urllib2.urlopen(self.sfFile)
            sData = oFile.read(1000)
            sFileEnc = guess_encoding(sData, self.sfFile)
            oFile.close()
            return codecs.EncodedFile(urllib2.urlopen(self.sfFile), 'utf8',
                    sFileEnc)
        else:
            oFile = open(self.sfFile, 'rU')
            sData = oFile.read(1000)
            sFileEnc = guess_encoding(sData, self.sfFile)
            oFile.close()
            return codecs.EncodedFile(open(self.sfFile, 'rU'), 'utf8',
                    sFileEnc)



# LackeyDeckParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Lackey Deck File Parser
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Parser for Lackey CCG deck format"""

# pylint: disable-msg=W0402
# string.digits is OK
import string
from sutekh.core.SutekhObjects import AbstractCard
from sutekh.io.WriteLackeyCCG import lackey_name
from sutekh.io.IOBase import BaseLineParser


def gen_name_lookups():
    """Create a lookup table to map Lackey CCG names to Sutekh names -
       reduces the number of user queries"""
    dNameCache = {}
    for oCard in AbstractCard.select():
        sSutekhName = oCard.name
        sLackeyName = lackey_name(oCard)
        if sLackeyName != sSutekhName:
            # Since we will need to check wether a card is in the dictionary
            # anyway (missed cases, etc), there's no point in having the
            # identity entries
            dNameCache[sLackeyName] = sSutekhName
    return dNameCache


class LackeyDeckParser(BaseLineParser):
    """Parser for the Lackey Deck format."""

    def __init__(self):
        super(LackeyDeckParser, self).__init__()
        self._dNameCache = gen_name_lookups()

    def _feed(self, sLine, oHolder):
        """Read the line into the given CardSetHolder"""
        if sLine[0] in string.digits:
            # Split on any whitespace
            sNum, sName = sLine.split(None, 1)
            try:
                iNum = int(sNum)
            except ValueError:
                raise IOError("Illegal number %s for Lacket CCG deck"
                        % sNum)
            if sName in self._dNameCache:
                sName = self._dNameCache[sName]
        elif sLine != 'Crypt:':
            raise IOError("Illegal string %s for Lackey CCG deck" %
                    sLine)
        else:
            # Skip the 'Crypt:' line
            return
        # Lackey has no expansion info
        oHolder.add(iNum, sName, None)



# test_SLInventoryParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2009 Simon Cross <hodgestar+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test reading a card set from an Secret Library inventory file"""

import unittest
import StringIO
from sutekh.tests.TestCore import SutekhTest, DummyHolder
from sutekh.io.SLInventoryParser import SLInventoryParser


class TestSLInventoryParser(SutekhTest):
    """Class for the Secret Library inventory file parser tests"""

    sTestText1 = """
        ***SL***CRYPT***
        2;2;Abebe
        2;2;Alan Sovereign (Adv)
        4;3;Fran\xc3\xa7ois Warden Loehr
        ***SL***LIBRARY***
        1;1;Absimiliard's Army
        5;5;Ahriman's Demesne
        4;4;Atonement
        1;1;Textbook Damnation, The
        8;8;Watch Commander
        ***SL***ENDEXPORT***
        """

    def test_basic(self):
        """Run the Secret Library inventory input test."""
        oHolder = DummyHolder()
        fIn = StringIO.StringIO(self.sTestText1)
        oParser = SLInventoryParser()
        oParser.parse(fIn, oHolder)

        aCards = oHolder.get_cards()

        self.assertEqual(len(aCards), 8)
        self.failUnless(("Abebe", 2) in aCards)
        self.failUnless(("Alan Sovereign (Advanced)", 2) in aCards)
        self.failUnless(("Fran\xc3\xa7ois Warden Loehr", 4) in aCards)
        self.failUnless(("Absimiliard's Army", 1) in aCards)
        self.failUnless(("Ahriman's Demesne", 5) in aCards)
        self.failUnless(("Atonement", 4) in aCards)
        self.failUnless(("The Textbook Damnation", 1) in aCards)
        self.failUnless(("Watch Commander", 8) in aCards)

if __name__ == "__main__":
    unittest.main()



# test_DatabaseUpgrade.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Test database upgrading"""

import sys
from sutekh.tests.TestCore import SutekhTest, make_null_handler
from sutekh.tests import create_db
from sutekh.core.DatabaseUpgrade import copy_to_new_abstract_card_db, \
                                        create_final_copy
from sutekh.core.CardLookup import SimpleLookup
from sutekh.core.SutekhObjects import AbstractCard, PhysicalCardSet, \
    AbstractCardAdapter, PhysicalCardAdapter, ExpansionAdapter, \
    IPhysicalCardSet
from sqlobject import sqlhub, connectionForURI


class DatabaseUpgradeTests(SutekhTest):
    """Class for the database upgrade tests."""

    def test_copy_to_new_ac_db(self):
        """Test copying an existing database to a freshly created one using
           copy_to_new_abstract_card_db."""
        # Create some database content

        oMyCollection = PhysicalCardSet(name="My Collection")
        oMyCollection.comment = "test comment"
        oMyCollection.author = "test author"

        oPCS1 = PhysicalCardSet(name="PCS1", parent=oMyCollection)
        # pylint: disable-msg=E1101
        # SQLObject confuses pylint

        oPC = PhysicalCardAdapter((AbstractCardAdapter(".44 magnum"),
            ExpansionAdapter("Jyhad")))
        oMyCollection.addPhysicalCard(oPC)

        assert list(PhysicalCardSet.select())

        # Attempt upgrade

        oOrigConn = sqlhub.processConnection

        sDbFile = self._create_tmp_file()
        # windows is different, since we don't have a starting / for the path
        if sys.platform.startswith("win"):
            oNewConn = connectionForURI("sqlite:///%s" % sDbFile)
        else:
            oNewConn = connectionForURI("sqlite://%s" % sDbFile)
        sqlhub.processConnection = oNewConn

        create_db()

        assert list(AbstractCard.select())

        oCardLookup = SimpleLookup()
        oLogHandler = make_null_handler()

        copy_to_new_abstract_card_db(oOrigConn, oNewConn, oCardLookup,
                oLogHandler)

        assert list(AbstractCard.select())
        assert list(PhysicalCardSet.select())

        sqlhub.processConnection = oOrigConn
        bResult, _aMsgs = create_final_copy(oNewConn, oLogHandler)

        # Check
        self.failUnless(bResult)

        assert list(AbstractCard.select())
        assert list(PhysicalCardSet.select())

        oMagnum = AbstractCardAdapter('.44 magnum')
        assert oMagnum

        oMyCollection = IPhysicalCardSet("My Collection")
        assert oMyCollection.comment == "test comment"
        assert oMyCollection.author == "test author"
        assert list(oMyCollection.cards)

        oPCS1 = IPhysicalCardSet("PCS1")
        assert oPCS1.parent == oMyCollection
        oNewConn.close()



# SecretLibrary.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar+sutekh@gmail.com>
# GPL - see COPYING for details

"""Plugin for interacting with the Secret Library website."""

import gtk
import urllib2
import urllib
import StringIO
import re
import unicodedata
from sutekh.SutekhInfo import SutekhInfo
from sutekh.core.SutekhObjects import PhysicalCardSet, IAbstractCard, \
    canonical_to_csv
from sutekh.core.Filters import CryptCardFilter, FilterNot
from sutekh.io.SLDeckParser import SLDeckParser
from sutekh.io.SLInventoryParser import SLInventoryParser
from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_error
from sutekh.gui.GuiCardSetFunctions import import_cs
from sutekh.gui.PluginManager import SutekhPlugin


def canonical_to_sl(sName):
    """Convert a canonical card name to a Secret Library name."""
    sName = canonical_to_csv(sName)
    return unicodedata.normalize('NFKD', sName).encode('ascii', 'ignore')


class ImportExportBase(SutekhDialog):
    # pylint: disable-msg=R0904, R0902
    # R0904 - gtk Widget, so has many public methods
    # R0902 - we use a lot of attributes to pass the data around
    """Base class for import and export dialogs."""

    _sCachedUsername = None
    _sCachedPassword = None

    # pylint: disable-msg=W0201
    # we define attributes outside __init__, but it's OK because of plugin
    # structure
    def _setup_vbox(self, sTitlePhrase, sSourcePhrase, aDeckWidgets,
                    aInvWidgets):
        """Set up Secret Library configuration dialog."""

        # pylint: disable-msg=E1101
        # pylint doesn't pick up vbox methods correctly

        self.vbox.set_spacing(10)

        oDescLabel = gtk.Label()
        oDescLabel.set_markup('<b>%s Secret Library</b>' % (sTitlePhrase,))

        self.vbox.pack_start(oDescLabel, False, False)

        # Deck / Inventory Selector

        self._sNewName = ""
        self._oAsDeckButton = gtk.RadioButton(None, "Deck")
        self._oAsInvButton = gtk.RadioButton(self._oAsDeckButton, "Inventory")
        self._oAsDeckButton.connect("toggled", self._deck_inv_changed)

        self.vbox.pack_start(
            gtk.Label("%s ..." % (sSourcePhrase,)), False, False)
        self.vbox.pack_start(self._oAsDeckButton, False, False)
        self.vbox.pack_start(self._oAsInvButton, False, False)

        # Deck / Inventory specific widgets

        self._oDeckInvVbox = gtk.VBox()
        self._aDeckWidgets = aDeckWidgets
        self._aInvWidgets = aInvWidgets
        self.vbox.pack_start(self._oDeckInvVbox, False, False)
        self._deck_inv_changed(self._oAsDeckButton)

        # API URL

        self._oUrlEntry = gtk.combo_box_entry_new_text()
        self._oUrlEntry.append_text(SecretLibrary.SL_API_URL)
        self._oUrlEntry.set_active(0)

        self.vbox.pack_start(gtk.Label("Secret Library API URL"), False, False)
        self.vbox.pack_start(self._oUrlEntry, False, False)

        # Username

        self._oUsernameEntry = gtk.Entry()
        if self._sCachedUsername:
            self._oUsernameEntry.set_text(self._sCachedUsername)

        self.vbox.pack_start(gtk.Label("Username"), False, False)
        self.vbox.pack_start(self._oUsernameEntry)

        # Password

        self._oPasswordEntry = gtk.Entry()
        self._oPasswordEntry.set_visibility(False)
        if self._sCachedPassword:
            self._oPasswordEntry.set_text(self._sCachedPassword)

        self.vbox.pack_start(gtk.Label("Password"), False, False)
        self.vbox.pack_start(self._oPasswordEntry)

    # pylint: enable-msg=W0201

    def _deck_inv_changed(self, _oWidget):
        """Handle a change in whether we're importing/exporting to/from a
           deck or an inventory.
           """
        if self._oAsDeckButton.get_active():
            aNewChildren = self._aDeckWidgets
        else:
            aNewChildren = self._aInvWidgets

        for oWidget in self._oDeckInvVbox.get_children():
            self._oDeckInvVbox.remove(oWidget)

        for oWidget in aNewChildren:
            self._oDeckInvVbox.add(oWidget)

        self._oDeckInvVbox.show_all()

    def get_api_url(self):
        """Return the current API URL."""
        return self._oUrlEntry.get_active_text()

    def get_username(self):
        """Return the username entered."""
        ImportExportBase._sCachedUsername = self._oUsernameEntry.get_text()
        return self._sCachedUsername

    def get_password(self):
        """Return the password entered."""
        ImportExportBase._sCachedPassword = self._oPasswordEntry.get_text()
        return self._sCachedPassword

    def get_sl_type(self):
        """Return whether the card set should be treated as a Secret Library
           deck or inventory.
           """
        if self._oAsDeckButton.get_active():
            return "deck"
        else:
            return "inventory"


class ImportDialog(ImportExportBase):
    # pylint: disable-msg=R0904
    # R0904 - gtk Widget, so has many public methods
    """Dialog for importing card sets from the Secret Library."""

    def __init__(self, oParent):
        super(ImportDialog, self).__init__('Import from Secret Library',
                oParent, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_OK, gtk.RESPONSE_OK,
                 gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        self._oDeckId = gtk.Entry()
        aDeckWidgets = [
            gtk.Label("Deck Id Number"),
            self._oDeckId,
        ]

        self._setup_vbox("Import from", "Import from", aDeckWidgets, [])

        self.show_all()

    def get_deck_id(self):
        """Get the Secret Library Deck Id (if exporting as a deck)"""
        if self.get_sl_type() != "deck":
            return None
        return self._oDeckId.get_text()


class ExportDialog(ImportExportBase):
    # pylint: disable-msg=R0904
    # R0904 - gtk Widget, so has many public methods
    """Dialog for exporting cards sets to the Secret Library."""

    def __init__(self, oParent):
        super(ExportDialog, self).__init__('Export to Secret Library',
                oParent, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_OK, gtk.RESPONSE_OK,
                 gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

        self._oPublic = gtk.CheckButton("Make deck public")
        aDeckWidgets = [self._oPublic]

        self._oPurge = gtk.CheckButton("Purge inventory on upload")
        aInvWidgets = [self._oPurge]

        self._setup_vbox("Export to", "Export as", aDeckWidgets, aInvWidgets)

        self.show_all()

    def get_public(self):
        """Return True if the exported deck should be marked as public."""
        return self._oPublic.get_active()

    def get_purge_inventory(self):
        """Return True if the inventory should be purged before exporting."""
        return self._oPurge.get_active()


class SecretLibrary(SutekhPlugin):
    """Provides ability to export and import cards directly
       to and from the Secret Library.
       """

    dTableVersions = {PhysicalCardSet: (5, 6)}
    aModelsSupported = (PhysicalCardSet, "MainWindow")

    SL_USER_AGENT = "Sutekh Secret Library Plugin"
    SL_AGENT_VERSION = SutekhInfo.VERSION_STR

    SL_API_URL = "http://www.secretlibrary.info/api.php"

    def get_menu_item(self):
        """Register on the 'Export Card Set' or 'Import Card Set' Menu"""
        if not self.check_versions() or not self.check_model_type():
            return None

        if self.model is None:
            oMenuItem = gtk.MenuItem("Import from Secret Library")
            oMenuItem.connect("activate", self.make_import_dialog)
            sSection = 'Import Card Set'
        else:
            oMenuItem = gtk.MenuItem("Export to Secret Library")
            oMenuItem.connect("activate", self.make_export_dialog)
            sSection = 'Export Card Set'

        return (sSection, oMenuItem)

    def make_import_dialog(self, _oWidget):
        """Create the importing dialog"""
        oDialog = ImportDialog(self.parent)
        self.handle_import_response(oDialog)

    def make_export_dialog(self, _oWidget):
        """Create the exporting dialog"""
        oDialog = ExportDialog(self.parent)
        self.handle_export_response(oDialog)

    def handle_import_response(self, oImportDialog):
        """Handle the import dialog response."""
        iResponse = oImportDialog.run()

        if iResponse == gtk.RESPONSE_OK:
            if oImportDialog.get_sl_type() == "deck":
                dData = {
                    'sl_deck_retrieve': '1',
                    'sl_user_agent': self.SL_USER_AGENT,
                    'sl_agent_version': self.SL_AGENT_VERSION,
                    'username': oImportDialog.get_username(),
                    'password': oImportDialog.get_password(),
                    'deck_id': oImportDialog.get_deck_id(),
                }
                sUrl = oImportDialog.get_api_url()
                self._import_deck(sUrl, dData)
            else:
                dData = {
                    'sl_inventory_export': '1',
                    'sl_user_agent': self.SL_USER_AGENT,
                    'sl_agent_version': self.SL_AGENT_VERSION,
                    'username': oImportDialog.get_username(),
                    'password': oImportDialog.get_password(),
                }
                sUrl = oImportDialog.get_api_url()
                self._import_inventory(sUrl, dData)

        # get rid of the dialog
        oImportDialog.destroy()

    def handle_export_response(self, oExportDialog):
        """Handle the export dialog response."""
        iResponse = oExportDialog.run()

        if iResponse == gtk.RESPONSE_OK:
            if oExportDialog.get_sl_type() == "deck":
                dData = {
                    # title, author, description, crypt and library filled
                    # by self._export_deck
                    'sl_deck_submit': '1',
                    'sl_user_agent': self.SL_USER_AGENT,
                    'sl_agent_version': self.SL_AGENT_VERSION,
                    'username': oExportDialog.get_username(),
                    'password': oExportDialog.get_password(),
                    'public': oExportDialog.get_public() and '1' or '0',
                }
                sUrl = oExportDialog.get_api_url()
                self._export_deck(sUrl, dData)
            else:
                dData = {
                    # inventory_crypt and inventory_library filled in by
                    # self._export_inventory
                    'sl_inventory_import': '1',
                    'sl_user_agent': self.SL_USER_AGENT,
                    'sl_agent_version': self.SL_AGENT_VERSION,
                    'username': oExportDialog.get_username(),
                    'password': oExportDialog.get_password(),
                }
                if oExportDialog.get_purge_inventory():
                    dData['inventory_purge'] = '1'
                sUrl = oExportDialog.get_api_url()
                self._export_inventory(sUrl, dData)

        # get rid of the dialog
        oExportDialog.destroy()

    SL_ERROR_RE = re.compile("^(?P<code>\d+)\:.*$")

    def _check_sl_result(self, sLine):
        """Inspect the line for a Secret Library error report.
           Return True if no error is found, False otherwise. Pops up an
           error dialog to the user if an error is found.
           """
        oMatch = self.SL_ERROR_RE.match(sLine)
        if oMatch:
            if int(oMatch.group('code')) != 0:
                do_complaint_error(
                        "The Secret Library site returned the error:\n"
                        + sLine.strip())
                return False
        return True

    def _import_deck(self, sApiUrl, dData):
        """Import a Secret Library deck as a card set."""
        sData = urllib.urlencode(dData)
        oParser = SLDeckParser()

        oReq = urllib2.Request(url=sApiUrl, data=sData)
        fReq = urllib2.urlopen(oReq)
        try:
            fIn = StringIO.StringIO(fReq.read())
        finally:
            fReq.close()

        sFirstLine = fIn.readline()
        fIn.seek(0)

        if self._check_sl_result(sFirstLine):
            import_cs(fIn, oParser, self.parent)

    def _import_inventory(self, sApiUrl, dData):
        """Import a Secret Library inventory as a card set."""
        sData = urllib.urlencode(dData)
        oParser = SLInventoryParser()

        oReq = urllib2.Request(url=sApiUrl, data=sData)
        fReq = urllib2.urlopen(oReq)
        try:
            fIn = StringIO.StringIO(fReq.read())
        finally:
            fReq.close()

        sFirstLine = fIn.readline()
        fIn.seek(0)

        if self._check_sl_result(sFirstLine):
            import_cs(fIn, oParser, self.parent)

    def _export_deck(self, sApiUrl, dData):
        """Export a card set to the Secret Library as a deck."""
        oCardSet = self.get_card_set()

        dData['title'] = oCardSet.name
        dData['author'] = oCardSet.author
        dData['description'] = oCardSet.comment
        dData['crypt'], dData['library'] = self._cs_as_deck()

        sData = urllib.urlencode(dData)

        oReq = urllib2.Request(url=sApiUrl, data=sData)
        fReq = urllib2.urlopen(oReq)
        try:
            sResponse = fReq.read()
        finally:
            fReq.close()

        self._check_sl_result(sResponse)

    def _export_inventory(self, sApiUrl, dData):
        """Export a card set to the Secret Library as an inventory."""
        dData['inventory_crypt'], dData['inventory_library'] = \
                self._cs_as_inventory()

        sData = urllib.urlencode(dData)

        oReq = urllib2.Request(url=sApiUrl, data=sData)
        fReq = urllib2.urlopen(oReq)
        try:
            sResponse = fReq.read()
        finally:
            fReq.close()

        self._check_sl_result(sResponse)

    def _cs_as_deck(self):
        """Convert a card set to an SL (crypt, library) string pair."""
        # populate crypt
        aCrypt = []
        oCryptFilter = CryptCardFilter()
        oCryptIter = self.model.get_card_iterator(oCryptFilter)

        for oCard in oCryptIter:
            # pylint: disable-msg=E1101
            # E1101: PyProtocols confuses pylint
            oAbsCard = IAbstractCard(oCard)
            sCsvName = canonical_to_sl(oAbsCard.name)
            sCsvName = sCsvName.replace('(Advanced)', '(Adv)')
            aCrypt.append(sCsvName)

        # populate library
        aLibrary = []
        oLibraryFilter = FilterNot(oCryptFilter)
        oLibraryIter = self.model.get_card_iterator(oLibraryFilter)

        for oCard in oLibraryIter:
            # pylint: disable-msg=E1101
            # E1101: PyProtocols confuses pylint
            oAbsCard = IAbstractCard(oCard)
            aLibrary.append(canonical_to_sl(oAbsCard.name))

        return "\n".join(aCrypt), "\n".join(aLibrary)

    def _cs_as_inventory(self):
        """Convert a card set to an SL (inventory_crypt, inventory_library)
           string pair.
           """
        # populate crypt dict
        dCrypt = {}
        oCryptFilter = CryptCardFilter()
        oCryptIter = self.model.get_card_iterator(oCryptFilter)

        for oCard in oCryptIter:
            oAbsCard = IAbstractCard(oCard)
            dCrypt.setdefault(oAbsCard, 0)
            dCrypt[oAbsCard] += 1

        # populate library dict
        dLibrary = {}
        oLibraryFilter = FilterNot(oCryptFilter)
        oLibraryIter = self.model.get_card_iterator(oLibraryFilter)

        for oCard in oLibraryIter:
            oAbsCard = IAbstractCard(oCard)
            dLibrary.setdefault(oAbsCard, 0)
            dLibrary[oAbsCard] += 1

        # create crypt strings
        aCrypt = []
        for oAbsCard, iCnt in dCrypt.iteritems():
            sCsvName = canonical_to_sl(oAbsCard.name)
            sCsvName = sCsvName.replace('(Advanced)', '(Adv)')
            # Secret library doesn't like "'s here, it seems.
            sCsvName = sCsvName.replace('"', '')
            # TODO: populate wanted field sensibly.
            # HAVE; WANTED; Card Name
            aCrypt.append("%d;%d;%s" % (iCnt, iCnt, sCsvName))
        aCrypt.sort()

        # create library strings
        aLibrary = []
        for oAbsCard, iCnt in dLibrary.iteritems():
            sCsvName = canonical_to_sl(oAbsCard.name)
            # TODO: populate wanted field sensibly.
            # HAVE; WANTED; Card Name
            aLibrary.append("%d;%d;%s" % (iCnt, iCnt, sCsvName))

        return "\n".join(aCrypt), "\n".join(aLibrary)


plugin = SecretLibrary



# ELDBInventoryParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# ELDB Inventory Parser
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Parser for ELDB inventory format"""


import re
from sutekh.core.ELDBUtilities import gen_name_lookups
from sutekh.io.IOBase import BaseLineParser


class ELDBInventoryParser(BaseLineParser):
    """Parser for the ELDB Inventory format.

       The inventory file has no name info, so the holder name isn't changed
       by the parser."""

    # Should this be based on CSVParser?
    _oCardRe = re.compile(r'\s*"(?P<name>[^"]*)"\s*,\s*(?P<cnt>[0-9]+)')

    def __init__(self):
        super(ELDBInventoryParser, self).__init__()
        self._dNameCache = gen_name_lookups()

    def _feed(self, sLine, oHolder):
        """Handle line by line data"""
        # sLine is stripped by parse
        if sLine.startswith('"ELDB - Inv'):
            # Skip header
            return
        oMatch = self._oCardRe.match(sLine)
        if oMatch:
            iCnt = int(oMatch.group('cnt'))
            sName = oMatch.group('name').strip()
            if sName in self._dNameCache:
                sName = self._dNameCache[sName]
            if iCnt:
                # ELDB will create 0 entries
                oHolder.add(iCnt, sName, None)



# test_FilterModelPanes.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2010 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test parts of the filter editor dialog"""

from sutekh.tests.GuiSutekhTest import GuiSutekhTest
import unittest
import gtk
from sutekh.core.FilterParser import FilterParser
from sutekh.gui.FilterModelPanes import FilterModelPanes, \
        FilterEditorToolbar, ENTER_KEYS
from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
from sutekh.gui.CardSetsListView import CardSetsListView
from sutekh.core.SutekhObjects import AbstractCard


class DummyDialog(gtk.Dialog):
    """Dummy dialog for the filter pane tests"""
    # pylint: disable-msg=R0904
    # gtk widget, so has many public methods

    def __init__(self):
        super(DummyDialog, self).__init__('Dummy', None, 0)
        # pylint: disable-msg=C0103
        # Needs to match the property name in the FilterDialog
        self.accel_group = gtk.AccelGroup()


class TestFilterModelPane(GuiSutekhTest):
    """Class for the FilterModelPanes tests"""

    def _check_asts(self, oAST1, oAST2):
        """Helper utility to check that two AST's generate identical
           results"""
        oFilter1 = oAST1.get_filter()
        oFilter2 = oAST2.get_filter()
        # We're not doing the PhysicalCard join, so select on abstract
        # card
        aResults1 = list(oFilter1.select(AbstractCard))
        aResults2 = list(oFilter2.select(AbstractCard))
        self.assertEqual(aResults1, aResults2, 'results differ: '
                '%s (from %s) != %s (from %s)' % (
                    aResults1, oAST1, aResults2, oAST2))

    def test_basic(self):
        """Set of simple tests of central part the Filter editor dialog"""
        # pylint: disable-msg=W0212, R0915
        # W0212: We directly access lots of internals details for testing
        # setup
        # R0915: Long test case to avoid repeated setups
        oParser = FilterParser()
        oDialog = DummyDialog()
        oFilterPanes = FilterModelPanes('PhysicalCard', oDialog)
        # check that we can round trip a filter with values
        oAST = oParser.apply('CardType in Equipment, Imbued or '
                '(Discipline in Presence and Sect in Sabbat)')
        oFilterPanes.replace_ast(oAST)
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oAST, oFilterAST)
        # Check that we can disable parts of the filter (sect subfilter)
        oAST2 = oParser.apply('CardType in Equipment, Imbued or '
                'Discipline in Presence')
        # select sect
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:1:1')
        oFilterPanes._oSelectBar._oBoxModelEditor.set_disabled(True)
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oAST2, oFilterAST)
        # Check that we can re-enable bits
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:1:1')
        oFilterPanes._oSelectBar._oBoxModelEditor.set_disabled(False)
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oAST, oFilterAST)

        # Check that we can negate bits
        oAST = oParser.apply('NOT CardType in Equipment, Imbued or '
                '(Discipline in Presence and Sect in Sabbat)')
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        oFilterPanes._oSelectBar._oBoxModelEditor.set_negate(True)
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oAST, oFilterAST)
        # reset
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        oFilterPanes._oSelectBar._oBoxModelEditor.set_negate(False)

        # Check that we can delete a value
        oAST = oParser.apply('CardType in Equipment or '
                '(Discipline in Presence and Sect in Sabbat)')
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0:1')
        oFilterPanes._oEditBox._oTreeView.set_cursor('0:0:1')
        oFilterPanes._oSelectBar._oBoxModelEditor.delete(None)
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oAST, oFilterAST)

        # Check that a filter without values is disabled.
        oAST = oParser.apply('Discipline in Presence and Sect in Sabbat')
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0:0')
        oFilterPanes._oEditBox._oTreeView.set_cursor('0:0:0')
        oFilterPanes._oSelectBar._oBoxModelEditor.delete(None)
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oAST, oFilterAST)

        # Check we can add filter values
        oAST = oParser.apply('CardType in Equipment or '
                '(Discipline in Presence and Sect in Sabbat)')
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        oFilterPanes._oEditBox._oTreeView.set_cursor('0:0')
        oWidget = oFilterPanes._oSelectBar._oWidget.get_children()[0]
        # check we can round trip setting a discipline on the list
        oList = oWidget.get_child()
        oList.set_selected_entry('Equipment')
        oFilter = oFilterPanes._oSelectBar._oLastFilter
        oEvent = gtk.gdk.Event(gtk.gdk.KEY_PRESS)
        # ENTER_KEYS contains longs, but keyval needs int
        oEvent.keyval = int(list(ENTER_KEYS)[0])
        oEvent.state = gtk.gdk.CONTROL_MASK
        oFilterPanes._oSelectBar.key_press(oFilterPanes._oSelectBar._oWidget,
                oEvent, oFilter, 'Value')
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oAST, oFilterAST)

        # Check that we can delete a filter element with values
        oAST = oParser.apply('Discipline in Presence and Sect in Sabbat')
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        oFilterPanes._oEditBox._oTreeView.set_cursor('0:0')
        oFilterPanes._oSelectBar._oBoxModelEditor.delete(None)
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oAST, oFilterAST)

        # Check adding filter elements and filter values
        oAST = oParser.apply('CardType in Equipment or '
                '(Discipline in Presence and Sect in Sabbat)')
        oFilterPanes.replace_ast(oAST)
        oAST = oParser.apply('CardType in Equipment or Keyword in location or '
                '(Discipline in Presence and Sect in Sabbat)')
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0')
        oFilterPanes._oEditBox._oTreeView.set_cursor('0')
        oWidget = oFilterPanes._oSelectBar._oWidget.get_children()[2]
        # check we can round trip setting a discipline on the list
        oToolbar = oWidget.get_child()
        oListStore = oToolbar._oListStore
        oSelection = oToolbar.get_selection()
        oIter = oListStore.get_iter_root()
        while oIter:
            if oListStore.get_value(oIter, 0) == 'Keyword':
                oSelection.select_path(oListStore.get_path(oIter))
                break
            oIter = oListStore.iter_next(oIter)
        oFilter = oFilterPanes._oSelectBar._oLastFilter
        # ENTER_KEYS contains longs, but keyval needs int
        oFilterPanes._oSelectBar.key_press(oToolbar, oEvent, oFilter, 'Filter')
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:2')
        oFilterPanes._oEditBox._oTreeView.set_cursor('0:2')
        oWidget = oFilterPanes._oSelectBar._oWidget.get_children()[0]
        # check we can round trip setting a discipline on the list
        oList = oWidget.get_child()
        oList.set_selected_entry('location')
        oFilter = oFilterPanes._oSelectBar._oLastFilter
        oFilterPanes._oSelectBar.key_press(oFilterPanes._oSelectBar._oWidget,
                oEvent, oFilter, 'Value')
        # ENTER_KEYS contains longs, but keyval needs int
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oAST, oFilterAST)

    def test_quoting(self):
        """Test that quotes are properly escaped from the gui widget"""
        # pylint: disable-msg=W0212, R0915
        # W0212: We directly access lots of internals details for testing
        # setup
        # R0915: Long test case to avoid repeated setups
        oParser = FilterParser()
        oDialog = DummyDialog()
        oFilterPanes = FilterModelPanes('PhysicalCard', oDialog)
        oAST = oParser.apply('CardName in $x')
        oFilterPanes.replace_ast(oAST)
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        # check we have entry widget
        oEntry = oFilterPanes._oSelectBar._oWidget.get_children()[1]

        oEntry.set_text('"')
        oFilterAST = oFilterPanes.get_ast_with_values()
        oDoubleQuoteAST = oParser.apply('CardName in \'"\'')
        self._check_asts(oDoubleQuoteAST, oFilterAST)

        oEntry.set_text("'")
        oFilterAST = oFilterPanes.get_ast_with_values()
        oSingleQuoteAST = oParser.apply('CardName in "\'"')
        self._check_asts(oSingleQuoteAST, oFilterAST)

        # Check quotes round trip properly
        oFilterPanes.replace_ast(oSingleQuoteAST)
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oSingleQuoteAST, oFilterAST)
        oFilterPanes.replace_ast(oDoubleQuoteAST)
        oFilterAST = oFilterPanes.get_ast_with_values()
        self._check_asts(oDoubleQuoteAST, oFilterAST)

    def test_widgets(self):
        """Test that various filters lead to the right type of widget"""
        # pylint: disable-msg=W0212
        # We directly access lots of internals details for testing
        oParser = FilterParser()
        oDialog = DummyDialog()
        oFilterPanes = FilterModelPanes('PhysicalCardSet', oDialog)
        # Check for none filter
        oAST = oParser.apply('CSSetsInUse')
        oFilterPanes.replace_ast(oAST)
        # Start with the filter box model selected
        # Label is the 2nd child
        oLabel = oFilterPanes._oSelectBar._oWidget.get_children()[1]
        self.assertEqual(oLabel.get_text(), "Or Drag Filter Element")
        oWidget = oFilterPanes._oSelectBar._oWidget.get_children()[2]
        self.assertTrue(isinstance(oWidget.get_child(), FilterEditorToolbar))
        # unselect
        oFilterPanes._oEditBox._oTreeView.get_selection().unselect_all()
        self.assertEqual(oFilterPanes._oSelectBar._oWidget,
                oFilterPanes._oSelectBar._oEmptyWidget)
        # select card set in use filter
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        self.assertEqual(oFilterPanes._oSelectBar._oWidget,
                oFilterPanes._oSelectBar._oNoneWidget)
        oAST = oParser.apply('CardSetAuthor in $x')
        oFilterPanes.replace_ast(oAST)
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        # check we have entry widget
        oEntry = oFilterPanes._oSelectBar._oWidget.get_children()[1]
        self.assertTrue(isinstance(oEntry, gtk.Entry))
        # change over to physical card filters
        oFilterPanes = FilterModelPanes('PhysicalCard', oDialog)
        oAST = oParser.apply('CardCount in $x')
        oFilterPanes.replace_ast(oAST)
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        # check we have a count filter
        oLabel = oFilterPanes._oSelectBar._oWidget.get_children()[1]
        self.assertEqual(oLabel.get_text(), "From")
        oWidget = oFilterPanes._oSelectBar._oWidget.get_children()[2]
        self.assertTrue(isinstance(oWidget.get_child(), CardSetsListView))

        oAST = oParser.apply('Card_Sets in $x')
        oFilterPanes.replace_ast(oAST)
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        # check we have a card set list filter
        oWidget = oFilterPanes._oSelectBar._oWidget.get_children()[0]
        self.assertTrue(isinstance(oWidget, CardSetsListView))

        oAST = oParser.apply('Discipline in $x')
        oFilterPanes.replace_ast(oAST)
        oFilterPanes._oEditBox._oTreeView.get_selection().select_path('0:0')
        # check we have a values list filter
        oWidget = oFilterPanes._oSelectBar._oWidget.get_children()[0]
        self.assertTrue(isinstance(oWidget, AutoScrolledWindow))
        # check we can round trip setting a discipline on the list
        oList = oWidget.get_child()
        oList.set_selected_entry('Presence')
        self.assertEqual(oList.get_selected_data(), ['Presence'])


if __name__ == "__main__":
    unittest.main()



# DatabaseVersion.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Simon Cross <hodgestar@gmail.com>,
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details
# version management helper class

"""Versioning support for Sutekh"""

from sqlobject import sqlhub
from sutekh.core.SutekhObjects import VersionTable


def _get_connection(oConn):
    """Ensure we have a valid connection object"""
    if oConn is None:
        return sqlhub.processConnection
    else:
        return oConn


class DatabaseVersion(object):
    """Class to handle all the database manipulation aspects."""

    _dCache = {}
    _dConns = {}

    def __init__(self, oConn=None):
        oConn = _get_connection(oConn)
        # We need to make sure the version table exists, even if
        # the rest of the database doesn't, otherwise early checks are
        # problematic
        self.ensure_table_exists(oConn)
        self.populate_cache(oConn)

    @classmethod
    def get_cache(cls):
        """Get a reference to the class-global results cache"""
        return cls._dCache

    @classmethod
    def expire_cache(cls):
        """Expire the results cache - needed for database upgrade code
           paths."""
        cls._dCache = {}

    @classmethod
    def populate_cache(cls, oConn):
        """Populate the results cache if empty - reduces queries at start
           up."""
        dCache = cls.get_cache()
        if dCache:
            return
        for oVer in VersionTable.select(connection=oConn):
            if oVer.TableName in dCache:
                raise RuntimeError("Multiple version entries for %s" \
                    " found in the database." % oVer.TableName)
            else:
                dCache[oVer.TableName] = oVer.Version

    @classmethod
    def ensure_table_exists(cls, oConn):
        """Enusre the table exists, but that describe is only called
           once per connection."""
        if not cls._dConns.has_key(oConn):
            VersionTable.createTable(ifNotExists=True, connection=oConn)
            cls._dConns[oConn] = True

    @classmethod
    def expire_table_conn(cls, oConn):
        """Expire the created table for this connection.
           Needed in the database upgrade code.
           """
        if not cls._dConns.has_key(oConn):
            return  # Nothing to do
        del cls._dConns[oConn]

    def set_version(self, oTable, iTableVersion, oConn=None):
        """Set the version for oTable to iTableVersion"""
        oConn = _get_connection(oConn)
        dCache = self.get_cache()
        try:
            sTableName = oTable.sqlmeta.table
        except AttributeError:
            return False
        aVer = list(VersionTable.selectBy(TableName=sTableName,
            connection=oConn))
        iTableCount = len(aVer)
        if iTableCount == 0:
            VersionTable(TableName=sTableName,
                         Version=iTableVersion, connection=oConn)
            dCache[sTableName] = iTableVersion
        elif iTableCount == 1:
            for oVersion in aVer:
                if oVersion.Version != iTableVersion:
                    VersionTable.delete(oVersion.id, connection=oConn)
                    VersionTable(TableName=sTableName,
                                 Version=iTableVersion, connection=oConn)
                dCache[sTableName] = iTableVersion
        else:
            print "Multiple version entries for %s in the database" \
                    % sTableName
            print "Giving up. I suggest dumping and reloading everything"
            return False
        return True

    def get_table_version(self, oTable, oConn=None):
        """Get the version number for oTable.

           returns -1 if no version info exists.
           """
        oConn = _get_connection(oConn)
        dCache = self.get_cache()
        iTableVersion = -1
        try:
            sName = oTable.sqlmeta.table
        except AttributeError:
            return iTableVersion
        if dCache.has_key(sName):
            return dCache[sName]
        aVer = list(VersionTable.selectBy(TableName=sName, connection=oConn))
        iTableCount = len(aVer)
        if iTableCount < 1:
            iTableVersion = -1
        elif iTableCount == 1:
            for oVersion in aVer:
                iTableVersion = oVersion.Version
                dCache[sName] = iTableVersion
        else:
            print "Multiple version entries for %s in the database" \
                    % oTable.sqlmeta.table
            print "Giving up. I suggest dumping and reloading everything"
            # Should this be an exception?
            iTableVersion = -999
        return iTableVersion

    def check_tables_and_versions(self, aTable, aTableVersion, oConn=None):
        """Check version numbers.

           aTable is the list of tables to check
           aTableVersion is a list of version numbers to match.
           It's assumed that aTable and aTableVersions are sorted the same.
           """
        bRes = True
        for oTable, iVersion in zip(aTable, aTableVersion):
            bRes = bRes and \
                    self.get_table_version(oTable, oConn) == iVersion
        return bRes

    def check_table_in_versions(self, oTable, aTableVersions, oConn=None):
        """Check version for a single table.

           Checks whether the given table has a version in the list of
           version numbers in the list aTableVersions.
           """
        iVersion = self.get_table_version(oTable, oConn)
        return iVersion in aTableVersions

    def get_bad_tables(self, aTable, aTableVersion, oConn=None):
        """Get tables that don't match the list of version numbers

           aTable is the list of tables to check
           aTableVersion is a list of version numbers to match.
           It's assumed that aTable and aTableVersions are sorted the same.
           Returns two lists, aLowerTables and aHigherTables, where
           aLowerTables is those with a lower version number, and
           aHigherTables is those with a higher number.
           """
        oConn = _get_connection(oConn)
        aLowerTables = []
        aHigherTables = []
        for oTable, iVersion in zip(aTable, aTableVersion):
            iCurVersion = self.get_table_version(oTable, oConn)
            if iCurVersion < iVersion:
                # Current database is lower than requested
                aLowerTables.append(oTable.sqlmeta.table)
            elif iCurVersion > iVersion:
                # Current database is higher than requested
                aHigherTables.append(oTable.sqlmeta.table)
        return aLowerTables, aHigherTables



# test_PhysicalCardSet.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test the PhysicalCardSet object handling"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.core.SutekhObjects import PhysicalCardSet, IPhysicalCardSet, \
        MapPhysicalCardToPhysicalCardSet
from sutekh.tests.core.test_Filters import make_card
from sutekh.core.CardSetUtilities import delete_physical_card_set
from sqlobject import SQLObjectNotFound
import unittest

ABSTRACT_CARDS = ['.44 magnum', 'ak-47', 'abbot', 'abebe', 'abombwe']
CARD_EXPANSIONS = [('.44 magnum', 'Jyhad'),
        ('ak-47', 'LotN'),
        ('abbot', 'Third Edition'),
        ('abombwe', 'Legacy of Blood'),
        ('alan sovereign (advanced)', 'Promo-20051001'),
        ('the path of blood', 'LotN'),
        ('the siamese', 'BL'),
        ('inez "nurse216" villagrande', 'NoR'),
        ('Scapelli, the Family "Mechanic"', 'DS'),
        ("Aaron's Feeding Razor", "KoT")]
CARD_SET_NAMES = ['Test Set 1', 'Test Set 2', 'Test Set 3', 'Test Set 4']


def get_phys_cards():
    """Get Physical Cards for the given lists"""
    aAddedPhysCards = []
    for sName in ABSTRACT_CARDS:
        oPC = make_card(sName, None)
        aAddedPhysCards.append(oPC)
    for sName, sExpansion in CARD_EXPANSIONS:
        oPC = make_card(sName, sExpansion)
        aAddedPhysCards.append(oPC)
    return aAddedPhysCards


def make_set_1():
    """Make the first card set.

       Function as this is also used in the io tests.
       """

    # pylint: disable-msg=E1101
    # E1101: SQLObject + PyProtocols magic confuses pylint
    aAddedPhysCards = get_phys_cards()
    oPhysCardSet1 = PhysicalCardSet(name=CARD_SET_NAMES[0])
    oPhysCardSet1.comment = 'A test comment'
    oPhysCardSet1.author = 'A test author'
    for oCard in aAddedPhysCards:
        oPhysCardSet1.addPhysicalCard(oCard.id)
    oPhysCardSet1.addPhysicalCard(aAddedPhysCards[0])
    oPhysCardSet1.addPhysicalCard(aAddedPhysCards[0])
    oPhysCardSet1.syncUpdate()
    return oPhysCardSet1


class PhysicalCardSetTests(SutekhTest):
    """class for the Physical Card Set tests"""

    def test_physical_card_set(self):
        """Test physical card set object"""
        # pylint: disable-msg=E1101, R0915, R0914
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # R0915, R0914: Want a long, sequential test case to minimise
        # repeated setups, so it has lots of lines + variables
        aAddedPhysCards = get_phys_cards()
        # We have a physical card list, so create some physical card sets

        oPhysCardSet1 = PhysicalCardSet(name=CARD_SET_NAMES[2])

        # Add cards to the physical card sets

        for iLoop in range(5):
            oPhysCardSet1.addPhysicalCard(aAddedPhysCards[iLoop].id)
        oPhysCardSet1.syncUpdate()

        self.assertEqual(len(oPhysCardSet1.cards), 5)
        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[0].id).count(), 1)
        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[4].id).count(), 1)
        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[7].id).count(), 0)

        oPhysCardSet2 = PhysicalCardSet(name=CARD_SET_NAMES[1],
                comment='Test 2', author=oPhysCardSet1.author)

        self.assertEqual(oPhysCardSet2.name, CARD_SET_NAMES[1])
        self.assertEqual(oPhysCardSet2.author, oPhysCardSet1.author)
        self.assertEqual(oPhysCardSet2.comment, 'Test 2')

        for iLoop in range(3, 8):
            oPhysCardSet2.addPhysicalCard(aAddedPhysCards[iLoop].id)
        oPhysCardSet2.syncUpdate()

        self.assertEqual(len(oPhysCardSet2.cards), 5)
        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[0].id).count(), 1)
        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[4].id).count(), 2)
        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[7].id).count(), 1)

        oPhysCardSet3 = make_set_1()
        self.assertEqual(len(oPhysCardSet3.cards), len(aAddedPhysCards) + 2)

        self.assertEqual(oPhysCardSet3.name, CARD_SET_NAMES[0])
        self.assertEqual(oPhysCardSet3.comment, 'A test comment')

        oPhysCardSet4 = IPhysicalCardSet(CARD_SET_NAMES[0])

        self.assertEqual(oPhysCardSet3, oPhysCardSet4)

        oPhysCardSet5 = PhysicalCardSet.byName(CARD_SET_NAMES[1])
        self.assertEqual(oPhysCardSet2, oPhysCardSet5)

        # Check Deletion

        for oCard in oPhysCardSet3.cards:
            oPhysCardSet3.removePhysicalCard(oCard.id)

        self.assertEqual(len(oPhysCardSet3.cards), 0)
        PhysicalCardSet.delete(oPhysCardSet3.id)

        self.assertRaises(SQLObjectNotFound, PhysicalCardSet.byName,
            CARD_SET_NAMES[0])

        delete_physical_card_set(CARD_SET_NAMES[1])

        self.assertRaises(SQLObjectNotFound, PhysicalCardSet.byName,
            CARD_SET_NAMES[1])

        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[0].id).count(), 1)
        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[4].id).count(), 1)

        delete_physical_card_set(CARD_SET_NAMES[2])

        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[0].id).count(), 0)
        self.assertEqual(MapPhysicalCardToPhysicalCardSet.selectBy(
            physicalCardID=aAddedPhysCards[4].id).count(), 0)

if __name__ == "__main__":
    unittest.main()



# PhysicalCardController.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2005, 2006 Simon Cross <hodgestar@gmail.com>
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Controller for the Physical Card Collection"""

from sutekh.gui.PhysicalCardView import PhysicalCardView


class PhysicalCardController(object):
    """Controller for the Physical Card Collection.

       Provide settings needed for the Physical Card List,
       and suitable card manipulation methods.
       """
    def __init__(self, oFrame, oMainWindow):
        self.__oMainWin = oMainWindow
        self.__oFrame = oFrame
        self.__oView = PhysicalCardView(self, oMainWindow,
                oMainWindow.config_file)
        self.__oModel = self.__oView.get_model()
        self._sFilterType = 'PhysicalCard'
        self.model.set_controller(self)

    # pylint: disable-msg=W0212
    # We provide read access to these items via these properties
    view = property(fget=lambda self: self.__oView, doc="Associated View")
    model = property(fget=lambda self: self.__oModel,
            doc="View's Model")
    frame = property(fget=lambda self: self.__oFrame, doc="Associated Frame")
    filtertype = property(fget=lambda self: self._sFilterType,
            doc="Associated Type")
    # pylint: enable-msg=W0212

    def cleanup(self):
        """Remove the signal handlers."""
        self.model.cleanup()

    def set_card_text(self, oCard):
        """Set the card text to reflect the selected card."""
        self.__oMainWin.set_card_text(oCard)

    def toggle_expansion(self, oWidget):
        """Toggle whether the expansion information is shown."""
        self.__oModel.bExpansions = oWidget.active
        self.__oView.reload_keep_expanded()

    def toggle_icons(self, oWidget):
        """Toggle the icons display"""
        self.__oModel.bUseIcons = oWidget.active
        self.__oView.reload_keep_expanded()



# test_IdentifyXMLFile.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test IdentifyXMLFile handling"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.io.IdentifyXMLFile import IdentifyXMLFile
from sutekh.tests.io.test_AbstractCardSetParser import ACS_EXAMPLE_1
from sutekh.tests.io.test_PhysicalCardSetParser import PCS_EXAMPLE_1
from sutekh.tests.io.test_PhysicalCardParser import make_example_pcxml
import unittest
from StringIO import StringIO


class TestIdentifyXMLFile(SutekhTest):
    """class for the IdentifyXMLFile tests"""

    def test_identify_xml_file(self):
        """Test IdentifyXMLFile"""
        # pylint: disable-msg=E1101
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # test IO
        sExample = make_example_pcxml()

        sTempFileName = self._create_tmp_file()
        fOut = open(sTempFileName, 'w')
        fOut.write(sExample)
        fOut.close()

        oIdFile = IdentifyXMLFile()
        oIdFile.id_file(sTempFileName)
        self.assertEqual(oIdFile.type, 'PhysicalCard')

        oIdFile.parse(StringIO(sExample), None)
        self.assertEqual(oIdFile.type, 'PhysicalCard')

        oIdFile.parse(StringIO('garbage input'))
        self.assertEqual(oIdFile.type, 'Unknown')

        oIdFile.parse(StringIO(ACS_EXAMPLE_1))
        self.assertEqual(oIdFile.type, 'AbstractCardSet')

        oIdFile.parse(StringIO(PCS_EXAMPLE_1))
        self.assertEqual(oIdFile.type, 'PhysicalCardSet')

if __name__ == "__main__":
    unittest.main()
